@@ -787,10 +787,6 @@ L&apos;ouverture d&apos;un nouveau fichier XML effacera les resultats. Voulez-vo
         <source>Unknown issue</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>C/C++ Source, Compile database, Visual Studio (%1 %2 *.sln *.vcxproj)</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Select configuration</source>
         <translation type="unfinished"></translation>
@@ -976,6 +972,53 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>Project files (*.cppcheck)</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Reanalyze and check library</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Check configuration (defines, includes)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>C/C++ Source</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Compile database</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Visual Studio</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Borland C++ Builder 6</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
+<context>
+    <name>NewSuppressionDialog</name>
+    <message>
+        <source>New suppression</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Error ID</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>File name</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Line number</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Symbol name</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>Platforms</name>
@@ -1101,10 +1144,6 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>Paths and Defines</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Import Project (Visual studio / compile database)</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;You have a choice:&lt;/p&gt;&lt;p&gt; * Analyze all Debug and Release configurations&lt;/p&gt;&lt;p&gt; * Only analyze the first matching Debug configuration&lt;/p&gt;&lt;p&gt;&lt;br/&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
         <translation type="unfinished"></translation>
@@ -1201,6 +1240,10 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>External tools</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Import Project (Visual studio / compile database/ Borland C++ Builder 6)</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ProjectFileDialog</name>
@@ -1220,14 +1263,6 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>Select a directory to check</source>
         <translation>Selectionner un répertoire à vérifier</translation>
     </message>
-    <message>
-        <source>Add Suppression</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Select error id suppress:</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Select Cppcheck build dir</source>
         <translation type="unfinished"></translation>
@@ -1236,10 +1271,6 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>Import Project</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Visual Studio (*.sln *.vcxproj);;Compile database (compile_commands.json)</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Clang-tidy (not found)</source>
         <translation type="unfinished"></translation>
@@ -1256,6 +1287,18 @@ Do you want to stop the analysis and exit Cppcheck?</source>
         <source>Misra rule texts file (%1)</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Visual Studio</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Compile database</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Borland C++ Builder 6</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>QDialogButtonBox</name>
@@ -1338,15 +1381,15 @@ Do you want to stop the analysis and exit Cppcheck?</source>
     </message>
     <message>
         <source>Copy filename</source>
-        <translation>Copier le nom du fichier</translation>
+        <translation type="vanished">Copier le nom du fichier</translation>
     </message>
     <message>
         <source>Copy full path</source>
-        <translation>Copier le chemin complet</translation>
+        <translation type="vanished">Copier le chemin complet</translation>
     </message>
     <message>
         <source>Copy message</source>
-        <translation>Copier le message</translation>
+        <translation type="vanished">Copier le message</translation>
     </message>
     <message>
         <source>Cppcheck</source>
@@ -1412,10 +1455,6 @@ Veuillez sélectionner le répertoire où est situé le fichier.</translation>
         <source>debug</source>
         <translation>débogage</translation>
     </message>
-    <message>
-        <source>[Inconclusive]</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>No editor application configured.
 
@@ -1434,7 +1473,7 @@ Please select the default editor application in preferences/Applications.</sourc
     </message>
     <message>
         <source>Copy message id</source>
-        <translation>Copier l&apos;identifiant du message</translation>
+        <translation type="vanished">Copier l&apos;identifiant du message</translation>
     </message>
     <message>
         <source>Hide all with id</source>
@@ -1472,10 +1511,6 @@ Please select the default editor application in preferences/Applications.</sourc
         <source>Since date</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Copy line nr</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Could not find file:</source>
         <translation type="unfinished"></translation>
@@ -1492,6 +1527,10 @@ Please select the default editor application in preferences/Applications.</sourc
         <source>Please select the directory where file is located.</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Copy</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ResultsView</name>
@@ -1995,4 +2034,15 @@ The user interface language has been reset to English. Open the Preferences-dial
         <translation type="unfinished"></translation>
     </message>
 </context>
+<context>
+    <name>toFilterString</name>
+    <message>
+        <source>All supported files (%1)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>All files (%1)</source>
+        <translation type="unfinished"></translation>
+    </message>
+</context>
 </TS>
