@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,10 +23,14 @@
 #include <QFile>
 #include <QDir>
 #include "projectfile.h"
+#include "common.h"
 
 static const char ProjectElementName[] = "project";
 static const char ProjectVersionAttrib[] = "version";
 static const char ProjectFileVersion[] = "1";
+static const char BuildDirElementName[] = "builddir";
+static const char ImportProjectElementName[] = "importproject";
+static const char AnalyzeAllVsConfigsElementName[] = "analyze-all-vs-configs";
 static const char IncludeDirElementName[] = "includedir";
 static const char DirElementName[] = "dir";
 static const char DirNameAttrib[] = "name";
@@ -46,21 +50,48 @@ static const char ExcludePathName[] = "path";
 static const char ExcludePathNameAttrib[] = "name";
 static const char LibrariesElementName[] = "libraries";
 static const char LibraryElementName[] = "library";
+static const char PlatformElementName[] = "platform";
 static const char SuppressionsElementName[] = "suppressions";
 static const char SuppressionElementName[] = "suppression";
+static const char AddonElementName[] = "addon";
+static const char AddonsElementName[] = "addons";
+static const char ToolElementName[] = "tool";
+static const char ToolsElementName[] = "tools";
+static const char TagsElementName[] = "tags";
+static const char TagElementName[] = "tag";
 
 ProjectFile::ProjectFile(QObject *parent) :
     QObject(parent)
 {
+    clear();
 }
 
 ProjectFile::ProjectFile(const QString &filename, QObject *parent) :
     QObject(parent),
     mFilename(filename)
 {
+    clear();
+    read();
 }
 
-bool ProjectFile::Read(const QString &filename)
+void ProjectFile::clear()
+{
+    mRootPath.clear();
+    mBuildDir.clear();
+    mImportProject.clear();
+    mAnalyzeAllVsConfigs = true;
+    mIncludeDirs.clear();
+    mDefines.clear();
+    mPaths.clear();
+    mExcludedPaths.clear();
+    mLibraries.clear();
+    mPlatform.clear();
+    mSuppressions.clear();
+    mAddons.clear();
+    mClangAnalyzer = mClangTidy = false;
+}
+
+bool ProjectFile::read(const QString &filename)
 {
     if (!filename.isEmpty())
         mFilename = filename;
@@ -69,6 +100,8 @@ bool ProjectFile::Read(const QString &filename)
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
         return false;
 
+    clear();
+
     QXmlStreamReader xmlReader(&file);
     bool insideProject = false;
     bool projectTagFound = false;
@@ -81,36 +114,64 @@ bool ProjectFile::Read(const QString &filename)
             }
             // Read root path from inside project element
             if (insideProject && xmlReader.name() == RootPathName)
-                ReadRootPath(xmlReader);
+                readRootPath(xmlReader);
+
+            // Read root path from inside project element
+            if (insideProject && xmlReader.name() == BuildDirElementName)
+                readBuildDir(xmlReader);
 
             // Find paths to check from inside project element
             if (insideProject && xmlReader.name() == PathsElementName)
-                ReadCheckPaths(xmlReader);
+                readCheckPaths(xmlReader);
+
+            if (insideProject && xmlReader.name() == ImportProjectElementName)
+                readImportProject(xmlReader);
+
+            if (insideProject && xmlReader.name() == AnalyzeAllVsConfigsElementName)
+                readAnalyzeAllVsConfigs(xmlReader);
 
             // Find include directory from inside project element
             if (insideProject && xmlReader.name() == IncludeDirElementName)
-                ReadIncludeDirs(xmlReader);
+                readIncludeDirs(xmlReader);
 
             // Find preprocessor define from inside project element
             if (insideProject && xmlReader.name() == DefinesElementName)
-                ReadDefines(xmlReader);
+                readDefines(xmlReader);
 
             // Find exclude list from inside project element
             if (insideProject && xmlReader.name() == ExcludeElementName)
-                ReadExcludes(xmlReader);
+                readExcludes(xmlReader);
 
             // Find ignore list from inside project element
             // These are read for compatibility
             if (insideProject && xmlReader.name() == IgnoreElementName)
-                ReadExcludes(xmlReader);
+                readExcludes(xmlReader);
 
             // Find libraries list from inside project element
             if (insideProject && xmlReader.name() == LibrariesElementName)
-                ReadStringList(mLibraries, xmlReader,LibraryElementName);
+                readStringList(mLibraries, xmlReader,LibraryElementName);
+
+            if (insideProject && xmlReader.name() == PlatformElementName)
+                readPlatform(xmlReader);
 
             // Find suppressions list from inside project element
             if (insideProject && xmlReader.name() == SuppressionsElementName)
-                ReadStringList(mSuppressions, xmlReader,SuppressionElementName);
+                readSuppressions(xmlReader);
+
+            // Addons
+            if (insideProject && xmlReader.name() == AddonsElementName)
+                readStringList(mAddons, xmlReader, AddonElementName);
+
+            // Tools
+            if (insideProject && xmlReader.name() == ToolsElementName) {
+                QStringList tools;
+                readStringList(tools, xmlReader, ToolElementName);
+                mClangAnalyzer = tools.contains(CLANG_ANALYZER);
+                mClangTidy = tools.contains(CLANG_TIDY);
+            }
+
+            if (insideProject && xmlReader.name() == TagsElementName)
+                readStringList(mTags, xmlReader, TagElementName);
 
             break;
 
@@ -119,7 +180,7 @@ bool ProjectFile::Read(const QString &filename)
                 insideProject = false;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -134,71 +195,92 @@ bool ProjectFile::Read(const QString &filename)
     }
 
     file.close();
-    if (projectTagFound)
-        return true;
-    else
-        return false;
+    return projectTagFound;
 }
 
-QStringList ProjectFile::GetIncludeDirs() const
-{
-    QStringList dirs;
-    foreach(QString path, mIncludeDirs) {
-        dirs << QDir::fromNativeSeparators(path);
-    }
-    return dirs;
-}
-
-QStringList ProjectFile::GetDefines() const
-{
-    return mDefines;
-}
-
-QStringList ProjectFile::GetCheckPaths() const
-{
-    QStringList paths;
-    foreach(QString path, mPaths) {
-        paths << QDir::fromNativeSeparators(path);
-    }
-    return paths;
-}
-
-QStringList ProjectFile::GetExcludedPaths() const
-{
-    QStringList paths;
-    foreach(QString path, mExcludedPaths) {
-        paths << QDir::fromNativeSeparators(path);
-    }
-    return paths;
-}
-
-QStringList ProjectFile::GetLibraries() const
-{
-    QStringList libraries;
-    foreach(QString library, mLibraries) {
-        libraries << library;
-    }
-    return libraries;
-}
-
-QStringList ProjectFile::GetSuppressions() const
-{
-    QStringList suppressions;
-    foreach(QString suppression, mSuppressions) {
-        suppressions << suppression;
-    }
-    return suppressions;
-}
-
-void ProjectFile::ReadRootPath(QXmlStreamReader &reader)
+void ProjectFile::readRootPath(QXmlStreamReader &reader)
 {
     QXmlStreamAttributes attribs = reader.attributes();
-    QString name = attribs.value("", RootPathNameAttrib).toString();
+    QString name = attribs.value(QString(), RootPathNameAttrib).toString();
     if (!name.isEmpty())
         mRootPath = name;
 }
 
-void ProjectFile::ReadIncludeDirs(QXmlStreamReader &reader)
+void ProjectFile::readBuildDir(QXmlStreamReader &reader)
+{
+    mBuildDir.clear();
+    do {
+        const QXmlStreamReader::TokenType type = reader.readNext();
+        switch (type) {
+        case QXmlStreamReader::Characters:
+            mBuildDir = reader.text().toString();
+        case QXmlStreamReader::EndElement:
+            return;
+        // Not handled
+        case QXmlStreamReader::StartElement:
+        case QXmlStreamReader::NoToken:
+        case QXmlStreamReader::Invalid:
+        case QXmlStreamReader::StartDocument:
+        case QXmlStreamReader::EndDocument:
+        case QXmlStreamReader::Comment:
+        case QXmlStreamReader::DTD:
+        case QXmlStreamReader::EntityReference:
+        case QXmlStreamReader::ProcessingInstruction:
+            break;
+        }
+    } while (1);
+}
+
+void ProjectFile::readImportProject(QXmlStreamReader &reader)
+{
+    mImportProject.clear();
+    do {
+        const QXmlStreamReader::TokenType type = reader.readNext();
+        switch (type) {
+        case QXmlStreamReader::Characters:
+            mImportProject = reader.text().toString();
+        case QXmlStreamReader::EndElement:
+            return;
+        // Not handled
+        case QXmlStreamReader::StartElement:
+        case QXmlStreamReader::NoToken:
+        case QXmlStreamReader::Invalid:
+        case QXmlStreamReader::StartDocument:
+        case QXmlStreamReader::EndDocument:
+        case QXmlStreamReader::Comment:
+        case QXmlStreamReader::DTD:
+        case QXmlStreamReader::EntityReference:
+        case QXmlStreamReader::ProcessingInstruction:
+            break;
+        }
+    } while (1);
+}
+
+void ProjectFile::readAnalyzeAllVsConfigs(QXmlStreamReader &reader)
+{
+    do {
+        const QXmlStreamReader::TokenType type = reader.readNext();
+        switch (type) {
+        case QXmlStreamReader::Characters:
+            mAnalyzeAllVsConfigs = (reader.text().toString() == "true");
+        case QXmlStreamReader::EndElement:
+            return;
+        // Not handled
+        case QXmlStreamReader::StartElement:
+        case QXmlStreamReader::NoToken:
+        case QXmlStreamReader::Invalid:
+        case QXmlStreamReader::StartDocument:
+        case QXmlStreamReader::EndDocument:
+        case QXmlStreamReader::Comment:
+        case QXmlStreamReader::DTD:
+        case QXmlStreamReader::EntityReference:
+        case QXmlStreamReader::ProcessingInstruction:
+            break;
+        }
+    } while (1);
+}
+
+void ProjectFile::readIncludeDirs(QXmlStreamReader &reader)
 {
     QXmlStreamReader::TokenType type;
     bool allRead = false;
@@ -210,7 +292,7 @@ void ProjectFile::ReadIncludeDirs(QXmlStreamReader &reader)
             // Read dir-elements
             if (reader.name().toString() == DirElementName) {
                 QXmlStreamAttributes attribs = reader.attributes();
-                QString name = attribs.value("", DirNameAttrib).toString();
+                QString name = attribs.value(QString(), DirNameAttrib).toString();
                 if (!name.isEmpty())
                     mIncludeDirs << name;
             }
@@ -221,7 +303,7 @@ void ProjectFile::ReadIncludeDirs(QXmlStreamReader &reader)
                 allRead = true;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -236,7 +318,7 @@ void ProjectFile::ReadIncludeDirs(QXmlStreamReader &reader)
     } while (!allRead);
 }
 
-void ProjectFile::ReadDefines(QXmlStreamReader &reader)
+void ProjectFile::readDefines(QXmlStreamReader &reader)
 {
     QXmlStreamReader::TokenType type;
     bool allRead = false;
@@ -247,7 +329,7 @@ void ProjectFile::ReadDefines(QXmlStreamReader &reader)
             // Read define-elements
             if (reader.name().toString() == DefineName) {
                 QXmlStreamAttributes attribs = reader.attributes();
-                QString name = attribs.value("", DefineNameAttrib).toString();
+                QString name = attribs.value(QString(), DefineNameAttrib).toString();
                 if (!name.isEmpty())
                     mDefines << name;
             }
@@ -258,7 +340,7 @@ void ProjectFile::ReadDefines(QXmlStreamReader &reader)
                 allRead = true;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -273,7 +355,7 @@ void ProjectFile::ReadDefines(QXmlStreamReader &reader)
     } while (!allRead);
 }
 
-void ProjectFile::ReadCheckPaths(QXmlStreamReader &reader)
+void ProjectFile::readCheckPaths(QXmlStreamReader &reader)
 {
     QXmlStreamReader::TokenType type;
     bool allRead = false;
@@ -285,7 +367,7 @@ void ProjectFile::ReadCheckPaths(QXmlStreamReader &reader)
             // Read dir-elements
             if (reader.name().toString() == PathName) {
                 QXmlStreamAttributes attribs = reader.attributes();
-                QString name = attribs.value("", PathNameAttrib).toString();
+                QString name = attribs.value(QString(), PathNameAttrib).toString();
                 if (!name.isEmpty())
                     mPaths << name;
             }
@@ -296,7 +378,7 @@ void ProjectFile::ReadCheckPaths(QXmlStreamReader &reader)
                 allRead = true;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -311,7 +393,7 @@ void ProjectFile::ReadCheckPaths(QXmlStreamReader &reader)
     } while (!allRead);
 }
 
-void ProjectFile::ReadExcludes(QXmlStreamReader &reader)
+void ProjectFile::readExcludes(QXmlStreamReader &reader)
 {
     QXmlStreamReader::TokenType type;
     bool allRead = false;
@@ -322,14 +404,14 @@ void ProjectFile::ReadExcludes(QXmlStreamReader &reader)
             // Read exclude-elements
             if (reader.name().toString() == ExcludePathName) {
                 QXmlStreamAttributes attribs = reader.attributes();
-                QString name = attribs.value("", ExcludePathNameAttrib).toString();
+                QString name = attribs.value(QString(), ExcludePathNameAttrib).toString();
                 if (!name.isEmpty())
                     mExcludedPaths << name;
             }
             // Read ignore-elements - deprecated but support reading them
             else if (reader.name().toString() == IgnorePathName) {
                 QXmlStreamAttributes attribs = reader.attributes();
-                QString name = attribs.value("", IgnorePathNameAttrib).toString();
+                QString name = attribs.value(QString(), IgnorePathNameAttrib).toString();
                 if (!name.isEmpty())
                     mExcludedPaths << name;
             }
@@ -342,7 +424,7 @@ void ProjectFile::ReadExcludes(QXmlStreamReader &reader)
                 allRead = true;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -357,8 +439,77 @@ void ProjectFile::ReadExcludes(QXmlStreamReader &reader)
     } while (!allRead);
 }
 
+void ProjectFile::readPlatform(QXmlStreamReader &reader)
+{
+    do {
+        const QXmlStreamReader::TokenType type = reader.readNext();
+        switch (type) {
+        case QXmlStreamReader::Characters:
+            mPlatform = reader.text().toString();
+        case QXmlStreamReader::EndElement:
+            return;
+        // Not handled
+        case QXmlStreamReader::StartElement:
+        case QXmlStreamReader::NoToken:
+        case QXmlStreamReader::Invalid:
+        case QXmlStreamReader::StartDocument:
+        case QXmlStreamReader::EndDocument:
+        case QXmlStreamReader::Comment:
+        case QXmlStreamReader::DTD:
+        case QXmlStreamReader::EntityReference:
+        case QXmlStreamReader::ProcessingInstruction:
+            break;
+        }
+    } while (1);
+}
 
-void ProjectFile::ReadStringList(QStringList &stringlist, QXmlStreamReader &reader, const char elementname[])
+
+void ProjectFile::readSuppressions(QXmlStreamReader &reader)
+{
+    QXmlStreamReader::TokenType type;
+    do {
+        type = reader.readNext();
+        switch (type) {
+        case QXmlStreamReader::StartElement:
+            // Read library-elements
+            if (reader.name().toString() == SuppressionElementName) {
+                Suppressions::Suppression suppression;
+                if (reader.attributes().hasAttribute(QString(),"fileName"))
+                    suppression.fileName = reader.attributes().value(QString(),"fileName").toString().toStdString();
+                if (reader.attributes().hasAttribute(QString(),"lineNumber"))
+                    suppression.lineNumber = reader.attributes().value(QString(),"lineNumber").toInt();
+                if (reader.attributes().hasAttribute(QString(),"symbolName"))
+                    suppression.symbolName = reader.attributes().value(QString(),"symbolName").toString().toStdString();
+                type = reader.readNext();
+                if (type == QXmlStreamReader::Characters) {
+                    suppression.errorId = reader.text().toString().toStdString();
+                }
+                mSuppressions << suppression;
+            }
+            break;
+
+        case QXmlStreamReader::EndElement:
+            if (reader.name().toString() != SuppressionElementName)
+                return;
+            break;
+
+        // Not handled
+        case QXmlStreamReader::NoToken:
+        case QXmlStreamReader::Invalid:
+        case QXmlStreamReader::StartDocument:
+        case QXmlStreamReader::EndDocument:
+        case QXmlStreamReader::Characters:
+        case QXmlStreamReader::Comment:
+        case QXmlStreamReader::DTD:
+        case QXmlStreamReader::EntityReference:
+        case QXmlStreamReader::ProcessingInstruction:
+            break;
+        }
+    } while (true);
+}
+
+
+void ProjectFile::readStringList(QStringList &stringlist, QXmlStreamReader &reader, const char elementname[])
 {
     QXmlStreamReader::TokenType type;
     bool allRead = false;
@@ -381,7 +532,7 @@ void ProjectFile::ReadStringList(QStringList &stringlist, QXmlStreamReader &read
                 allRead = true;
             break;
 
-            // Not handled
+        // Not handled
         case QXmlStreamReader::NoToken:
         case QXmlStreamReader::Invalid:
         case QXmlStreamReader::StartDocument:
@@ -396,37 +547,47 @@ void ProjectFile::ReadStringList(QStringList &stringlist, QXmlStreamReader &read
     } while (!allRead);
 }
 
-void ProjectFile::SetIncludes(const QStringList &includes)
+void ProjectFile::setIncludes(const QStringList &includes)
 {
     mIncludeDirs = includes;
 }
 
-void ProjectFile::SetDefines(const QStringList &defines)
+void ProjectFile::setDefines(const QStringList &defines)
 {
     mDefines = defines;
 }
 
-void ProjectFile::SetCheckPaths(const QStringList &paths)
+void ProjectFile::setCheckPaths(const QStringList &paths)
 {
     mPaths = paths;
 }
 
-void ProjectFile::SetExcludedPaths(const QStringList &paths)
+void ProjectFile::setExcludedPaths(const QStringList &paths)
 {
     mExcludedPaths = paths;
 }
 
-void ProjectFile::SetLibraries(const QStringList &libraries)
+void ProjectFile::setLibraries(const QStringList &libraries)
 {
     mLibraries = libraries;
 }
 
-void ProjectFile::SetSuppressions(const QStringList &suppressions)
+void ProjectFile::setPlatform(const QString &platform)
+{
+    mPlatform = platform;
+}
+
+void ProjectFile::setSuppressions(const QList<Suppressions::Suppression> &suppressions)
 {
     mSuppressions = suppressions;
 }
 
-bool ProjectFile::Write(const QString &filename)
+void ProjectFile::setAddons(const QStringList &addons)
+{
+    mAddons = addons;
+}
+
+bool ProjectFile::write(const QString &filename)
 {
     if (!filename.isEmpty())
         mFilename = filename;
@@ -447,9 +608,31 @@ bool ProjectFile::Write(const QString &filename)
         xmlWriter.writeEndElement();
     }
 
+    if (!mBuildDir.isEmpty()) {
+        xmlWriter.writeStartElement(BuildDirElementName);
+        xmlWriter.writeCharacters(mBuildDir);
+        xmlWriter.writeEndElement();
+    }
+
+    if (!mPlatform.isEmpty()) {
+        xmlWriter.writeStartElement(PlatformElementName);
+        xmlWriter.writeCharacters(mPlatform);
+        xmlWriter.writeEndElement();
+    }
+
+    if (!mImportProject.isEmpty()) {
+        xmlWriter.writeStartElement(ImportProjectElementName);
+        xmlWriter.writeCharacters(mImportProject);
+        xmlWriter.writeEndElement();
+    }
+
+    xmlWriter.writeStartElement(AnalyzeAllVsConfigsElementName);
+    xmlWriter.writeCharacters(mAnalyzeAllVsConfigs ? "true" : "false");
+    xmlWriter.writeEndElement();
+
     if (!mIncludeDirs.isEmpty()) {
         xmlWriter.writeStartElement(IncludeDirElementName);
-        foreach(QString incdir, mIncludeDirs) {
+        foreach (QString incdir, mIncludeDirs) {
             xmlWriter.writeStartElement(DirElementName);
             xmlWriter.writeAttribute(DirNameAttrib, incdir);
             xmlWriter.writeEndElement();
@@ -459,7 +642,7 @@ bool ProjectFile::Write(const QString &filename)
 
     if (!mDefines.isEmpty()) {
         xmlWriter.writeStartElement(DefinesElementName);
-        foreach(QString define, mDefines) {
+        foreach (QString define, mDefines) {
             xmlWriter.writeStartElement(DefineName);
             xmlWriter.writeAttribute(DefineNameAttrib, define);
             xmlWriter.writeEndElement();
@@ -469,7 +652,7 @@ bool ProjectFile::Write(const QString &filename)
 
     if (!mPaths.isEmpty()) {
         xmlWriter.writeStartElement(PathsElementName);
-        foreach(QString path, mPaths) {
+        foreach (QString path, mPaths) {
             xmlWriter.writeStartElement(PathName);
             xmlWriter.writeAttribute(PathNameAttrib, path);
             xmlWriter.writeEndElement();
@@ -479,7 +662,7 @@ bool ProjectFile::Write(const QString &filename)
 
     if (!mExcludedPaths.isEmpty()) {
         xmlWriter.writeStartElement(ExcludeElementName);
-        foreach(QString path, mExcludedPaths) {
+        foreach (QString path, mExcludedPaths) {
             xmlWriter.writeStartElement(ExcludePathName);
             xmlWriter.writeAttribute(ExcludePathNameAttrib, path);
             xmlWriter.writeEndElement();
@@ -487,31 +670,78 @@ bool ProjectFile::Write(const QString &filename)
         xmlWriter.writeEndElement();
     }
 
-    WriteStringList(xmlWriter,
+    writeStringList(xmlWriter,
                     mLibraries,
                     LibrariesElementName,
                     LibraryElementName);
 
-    WriteStringList(xmlWriter,
-                    mSuppressions,
-                    SuppressionsElementName,
-                    SuppressionElementName);
+    if (!mSuppressions.isEmpty()) {
+        xmlWriter.writeStartElement(SuppressionsElementName);
+        foreach (const Suppressions::Suppression &suppression, mSuppressions) {
+            xmlWriter.writeStartElement(SuppressionElementName);
+            if (!suppression.fileName.empty())
+                xmlWriter.writeAttribute("fileName", QString::fromStdString(suppression.fileName));
+            if (suppression.lineNumber > 0)
+                xmlWriter.writeAttribute("lineNumber", QString::number(suppression.lineNumber));
+            if (!suppression.symbolName.empty())
+                xmlWriter.writeAttribute("symbolName", QString::fromStdString(suppression.symbolName));
+            if (!suppression.errorId.empty())
+                xmlWriter.writeCharacters(QString::fromStdString(suppression.errorId));
+            xmlWriter.writeEndElement();
+        }
+        xmlWriter.writeEndElement();
+    }
+
+    writeStringList(xmlWriter,
+                    mAddons,
+                    AddonsElementName,
+                    AddonElementName);
+
+    QStringList tools;
+    if (mClangAnalyzer)
+        tools << CLANG_ANALYZER;
+    if (mClangTidy)
+        tools << CLANG_TIDY;
+    writeStringList(xmlWriter,
+                    tools,
+                    ToolsElementName,
+                    ToolElementName);
+
+    writeStringList(xmlWriter, mTags, TagsElementName, TagElementName);
 
     xmlWriter.writeEndDocument();
     file.close();
     return true;
 }
 
-void ProjectFile::WriteStringList(QXmlStreamWriter &xmlWriter, const QStringList &stringlist, const char startelementname[], const char stringelementname[])
+void ProjectFile::writeStringList(QXmlStreamWriter &xmlWriter, const QStringList &stringlist, const char startelementname[], const char stringelementname[])
 {
     if (stringlist.isEmpty())
         return;
 
     xmlWriter.writeStartElement(startelementname);
-    foreach(QString str, stringlist) {
+    foreach (QString str, stringlist) {
         xmlWriter.writeStartElement(stringelementname);
         xmlWriter.writeCharacters(str);
         xmlWriter.writeEndElement();
     }
     xmlWriter.writeEndElement();
 }
+
+QStringList ProjectFile::fromNativeSeparators(const QStringList &paths)
+{
+    QStringList ret;
+    foreach (const QString &path, paths)
+        ret << QDir::fromNativeSeparators(path);
+    return ret;
+}
+
+QStringList ProjectFile::getAddonsAndTools() const
+{
+    QStringList ret(mAddons);
+    if (mClangAnalyzer)
+        ret << CLANG_ANALYZER;
+    if (mClangTidy)
+        ret << CLANG_TIDY;
+    return ret;
+}
