@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,14 +32,19 @@
  * - CheckMemoryLeakStructMember checks allocation/deallocation of structs and struct members
  */
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "errorlogger.h"
+#include "tokenize.h"
 
 #include <list>
 #include <string>
 
-class Scope;
 class Function;
+class Scope;
+class Settings;
+class SymbolDatabase;
+class Token;
 class Variable;
 
 /// @addtogroup Core
@@ -69,13 +74,13 @@ private:
 
     /**
      * Report error. Similar with the function Check::reportError
-     * @param location the token where the error occurs
+     * @param tok the token where the error occurs
      * @param severity the severity of the bug
      * @param id type of message
      * @param msg text
      * @param cwe cwe number
      */
-    void reportErr(const Token *location, Severity::SeverityType severity, const std::string &id, const std::string &msg, unsigned int cwe) const;
+    void reportErr(const Token *tok, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe) const;
 
     /**
      * Report error. Similar with the function Check::reportError
@@ -85,7 +90,7 @@ private:
      * @param msg text
      * @param cwe cwe number
      */
-    void reportErr(const std::list<const Token *> &callstack, Severity::SeverityType severity, const std::string &id, const std::string &msg, unsigned int cwe) const;
+    void reportErr(const std::list<const Token *> &callstack, Severity::SeverityType severity, const std::string &id, const std::string &msg, const CWE &cwe) const;
 
 public:
     CheckMemoryLeak(const Tokenizer *t, ErrorLogger *e, const Settings *s)
@@ -95,15 +100,7 @@ public:
     /** @brief What type of allocation are used.. the "Many" means that several types of allocation and deallocation are used */
     enum AllocType { No, Malloc, New, NewArray, File, Fd, Pipe, OtherMem, OtherRes, Many };
 
-    void memoryLeak(const Token *tok, const std::string &varname, AllocType alloctype);
-
-    /**
-     * @brief Get type of deallocation at given position
-     * @param tok position
-     * @param varname variable name
-     * @return type of deallocation
-     */
-    AllocType getDeallocationType(const Token *tok, const std::string &varname) const;
+    void memoryLeak(const Token *tok, const std::string &varname, AllocType alloctype) const;
 
     /**
      * @brief Get type of deallocation at given position
@@ -116,7 +113,7 @@ public:
     /**
      * @brief Get type of allocation at given position
      */
-    AllocType getAllocationType(const Token *tok2, unsigned int varid, std::list<const Function*> *callstack = NULL) const;
+    AllocType getAllocationType(const Token *tok2, unsigned int varid, std::list<const Function*> *callstack = nullptr) const;
 
     /**
      * @brief Get type of reallocation at given position
@@ -157,7 +154,7 @@ public:
     void memleakUponReallocFailureError(const Token *tok, const std::string &varname) const;
 
     /** What type of allocated memory does the given function return? */
-    AllocType functionReturnType(const Function* func, std::list<const Function*> *callstack = NULL) const;
+    AllocType functionReturnType(const Function* func, std::list<const Function*> *callstack = nullptr) const;
 
     /** Function allocates pointed-to argument (a la asprintf)? */
     const char *functionArgAlloc(const Function *func, unsigned int targetpar, AllocType &allocType) const;
@@ -184,7 +181,7 @@ public:
 class CPPCHECKLIB CheckMemoryLeakInFunction : private Check, public CheckMemoryLeak {
 public:
     /** @brief This constructor is used when registering this class */
-    CheckMemoryLeakInFunction() : Check(myName()), CheckMemoryLeak(0, 0, 0), symbolDatabase(NULL) {
+    CheckMemoryLeakInFunction() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr), symbolDatabase(nullptr) {
     }
 
     /** @brief This constructor is used when running checks */
@@ -194,11 +191,11 @@ public:
         if (tokenizr)
             symbolDatabase = tokenizr->getSymbolDatabase();
         else
-            symbolDatabase = 0;
+            symbolDatabase = nullptr;
     }
 
     /** @brief run all simplified checks */
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
+    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) override {
         CheckMemoryLeakInFunction checkMemoryLeak(tokenizr, settings, errLog);
         checkMemoryLeak.checkReallocUsage();
         checkMemoryLeak.check();
@@ -291,18 +288,18 @@ public:
 
 private:
     /** Report all possible errors (for the --errorlist) */
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakInFunction c(0, settings, e);
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const override {
+        CheckMemoryLeakInFunction c(nullptr, settings, e);
 
-        c.memleakError(0, "varname");
-        c.resourceLeakError(0, "varname");
+        c.memleakError(nullptr, "varname");
+        c.resourceLeakError(nullptr, "varname");
 
-        c.deallocDeallocError(0, "varname");
-        c.deallocuseError(0, "varname");
-        c.mismatchSizeError(0, "sz");
-        std::list<const Token *> callstack;
+        c.deallocDeallocError(nullptr, "varname");
+        c.deallocuseError(nullptr, "varname");
+        c.mismatchSizeError(nullptr, "sz");
+        const std::list<const Token *> callstack;
         c.mismatchAllocDealloc(callstack, "varname");
-        c.memleakUponReallocFailureError(0, "varname");
+        c.memleakUponReallocFailureError(nullptr, "varname");
     }
 
     /**
@@ -317,7 +314,7 @@ private:
      * Get class information (--doc)
      * @return Wiki formatted information about this class
      */
-    std::string classInfo() const {
+    std::string classInfo() const override {
         return "Is there any allocated memory when a function goes out of scope\n";
     }
 
@@ -332,14 +329,14 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakInClass : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakInClass() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakInClass() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
     CheckMemoryLeakInClass(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
         : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
+    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) override {
         if (!tokenizr->isCPP())
             return;
 
@@ -358,17 +355,17 @@ private:
 
     void unsafeClassError(const Token *tok, const std::string &classname, const std::string &varname);
 
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakInClass c(0, settings, e);
-        c.publicAllocationError(0, "varname");
-        c.unsafeClassError(0, "class", "class::varname");
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const override {
+        CheckMemoryLeakInClass c(nullptr, settings, e);
+        c.publicAllocationError(nullptr, "varname");
+        c.unsafeClassError(nullptr, "class", "class::varname");
     }
 
     static std::string myName() {
         return "Memory leaks (class variables)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const override {
         return "If the constructor allocate memory then the destructor must deallocate it.\n";
     }
 };
@@ -379,14 +376,14 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakStructMember : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakStructMember() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakStructMember() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
     CheckMemoryLeakStructMember(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
         : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
+    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) override {
         CheckMemoryLeakStructMember checkMemoryLeak(tokenizr, settings, errLog);
         checkMemoryLeak.check();
     }
@@ -400,14 +397,14 @@ private:
 
     void checkStructVariable(const Variable * const variable);
 
-    void getErrorMessages(ErrorLogger * /*errorLogger*/, const Settings * /*settings*/) const {
+    void getErrorMessages(ErrorLogger * /*errorLogger*/, const Settings * /*settings*/) const override {
     }
 
     static std::string myName() {
         return "Memory leaks (struct members)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const override {
         return "Don't forget to deallocate struct members\n";
     }
 };
@@ -418,14 +415,14 @@ private:
 
 class CPPCHECKLIB CheckMemoryLeakNoVar : private Check, private CheckMemoryLeak {
 public:
-    CheckMemoryLeakNoVar() : Check(myName()), CheckMemoryLeak(0, 0, 0) {
+    CheckMemoryLeakNoVar() : Check(myName()), CheckMemoryLeak(nullptr, nullptr, nullptr) {
     }
 
     CheckMemoryLeakNoVar(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog)
         : Check(myName(), tokenizr, settings, errLog), CheckMemoryLeak(tokenizr, errLog, settings) {
     }
 
-    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) {
+    void runSimplifiedChecks(const Tokenizer *tokenizr, const Settings *settings, ErrorLogger *errLog) override {
         CheckMemoryLeakNoVar checkMemoryLeak(tokenizr, settings, errLog);
         checkMemoryLeak.check();
     }
@@ -449,19 +446,19 @@ private:
     void returnValueNotUsedError(const Token* tok, const std::string &alloc);
     void unsafeArgAllocError(const Token *tok, const std::string &funcName, const std::string &ptrType, const std::string &objType);
 
-    void getErrorMessages(ErrorLogger *e, const Settings *settings) const {
-        CheckMemoryLeakNoVar c(0, settings, e);
+    void getErrorMessages(ErrorLogger *e, const Settings *settings) const override {
+        CheckMemoryLeakNoVar c(nullptr, settings, e);
 
-        c.functionCallLeak(0, "funcName", "funcName");
-        c.returnValueNotUsedError(0, "funcName");
-        c.unsafeArgAllocError(0, "funcName", "shared_ptr", "int");
+        c.functionCallLeak(nullptr, "funcName", "funcName");
+        c.returnValueNotUsedError(nullptr, "funcName");
+        c.unsafeArgAllocError(nullptr, "funcName", "shared_ptr", "int");
     }
 
     static std::string myName() {
         return "Memory leaks (address not taken)";
     }
 
-    std::string classInfo() const {
+    std::string classInfo() const override {
         return "Not taking the address to allocated memory\n";
     }
 };
