@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,15 @@
  */
 
 
-
 #include "mathlib.h"
 #include "errorlogger.h"
+#include "utils.h"
 
-#include <cmath>
 #include <cctype>
+#include <cmath>
+#include <cstdlib>
 #include <limits>
-
+#include <locale>
 
 #if defined(_MSC_VER) && _MSC_VER <= 1700  // VS2012 doesn't have std::isinf and std::isnan
 #define ISINF(x)      (!_finite(x))
@@ -37,6 +38,7 @@
 #define ISNAN(x)      (std::isnan(x))
 #endif
 
+const int MathLib::bigint_bits = 64;
 
 MathLib::value::value(const std::string &s) :
     intValue(0), doubleValue(0), isUnsigned(false)
@@ -48,15 +50,18 @@ MathLib::value::value(const std::string &s) :
     }
 
     if (!MathLib::isInt(s))
-        throw InternalError(0, "Invalid value");
+        throw InternalError(nullptr, "Invalid value: " + s);
 
     type = MathLib::value::INT;
     intValue = MathLib::toLongNumber(s);
 
+    if (isIntHex(s) && intValue < 0)
+        isUnsigned = true;
+
     // read suffix
     if (s.size() >= 2U) {
         for (std::size_t i = s.size() - 1U; i > 0U; --i) {
-            char c = s[i];
+            const char c = s[i];
             if (c == 'u' || c == 'U')
                 isUnsigned = true;
             else if (c == 'l' || c == 'L') {
@@ -64,7 +69,8 @@ MathLib::value::value(const std::string &s) :
                     type = MathLib::value::LONG;
                 else if (type == MathLib::value::LONG)
                     type = MathLib::value::LONGLONG;
-            }
+            } else if (i > 2U && c == '4' && s[i-1] == '6' && s[i-2] == 'i')
+                type = MathLib::value::LONGLONG;
         }
     }
 }
@@ -92,9 +98,10 @@ std::string MathLib::value::str() const
         return ret.substr(0, pos+1);
     }
 
-    ostr << intValue;
     if (isUnsigned)
-        ostr << "U";
+        ostr << static_cast<biguint>(intValue) << "U";
+    else
+        ostr << intValue;
     if (type == MathLib::value::LONG)
         ostr << "L";
     else if (type == MathLib::value::LONGLONG)
@@ -141,9 +148,9 @@ MathLib::value MathLib::value::calc(char op, const MathLib::value &v1, const Mat
         case '&':
         case '|':
         case '^':
-            throw InternalError(0, "Invalid calculation");
+            throw InternalError(nullptr, "Invalid calculation");
         default:
-            throw InternalError(0, "Unhandled calculation");
+            throw InternalError(nullptr, "Unhandled calculation");
         }
     } else if (temp.isUnsigned) {
         switch (op) {
@@ -158,14 +165,14 @@ MathLib::value MathLib::value::calc(char op, const MathLib::value &v1, const Mat
             break;
         case '/':
             if (v2.intValue == 0)
-                throw InternalError(0, "Internal Error: Division by zero");
+                throw InternalError(nullptr, "Internal Error: Division by zero");
             if (v1.intValue == std::numeric_limits<bigint>::min() && std::abs(v2.intValue)<=1)
-                throw InternalError(0, "Internal Error: Division overflow");
+                throw InternalError(nullptr, "Internal Error: Division overflow");
             temp.intValue /= (unsigned long long)v2.intValue;
             break;
         case '%':
             if (v2.intValue == 0)
-                throw InternalError(0, "Internal Error: Division by zero");
+                throw InternalError(nullptr, "Internal Error: Division by zero");
             temp.intValue %= (unsigned long long)v2.intValue;
             break;
         case '&':
@@ -178,7 +185,7 @@ MathLib::value MathLib::value::calc(char op, const MathLib::value &v1, const Mat
             temp.intValue ^= (unsigned long long)v2.intValue;
             break;
         default:
-            throw InternalError(0, "Unhandled calculation");
+            throw InternalError(nullptr, "Unhandled calculation");
         }
     } else {
         switch (op) {
@@ -193,14 +200,14 @@ MathLib::value MathLib::value::calc(char op, const MathLib::value &v1, const Mat
             break;
         case '/':
             if (v2.intValue == 0)
-                throw InternalError(0, "Internal Error: Division by zero");
+                throw InternalError(nullptr, "Internal Error: Division by zero");
             if (v1.intValue == std::numeric_limits<bigint>::min() && std::abs(v2.intValue)<=1)
-                throw InternalError(0, "Internal Error: Division overflow");
+                throw InternalError(nullptr, "Internal Error: Division overflow");
             temp.intValue /= v2.intValue;
             break;
         case '%':
             if (v2.intValue == 0)
-                throw InternalError(0, "Internal Error: Division by zero");
+                throw InternalError(nullptr, "Internal Error: Division by zero");
             temp.intValue %= v2.intValue;
             break;
         case '&':
@@ -213,7 +220,7 @@ MathLib::value MathLib::value::calc(char op, const MathLib::value &v1, const Mat
             temp.intValue ^= v2.intValue;
             break;
         default:
-            throw InternalError(0, "Unhandled calculation");
+            throw InternalError(nullptr, "Unhandled calculation");
         }
     }
     return temp;
@@ -258,6 +265,30 @@ MathLib::value MathLib::value::add(int v) const
     return temp;
 }
 
+MathLib::value MathLib::value::shiftLeft(const MathLib::value &v) const
+{
+    if (!isInt() || !v.isInt())
+        throw InternalError(nullptr, "Shift operand is not integer");
+    MathLib::value ret(*this);
+    if (v.intValue >= MathLib::bigint_bits) {
+        return ret;
+    }
+    ret.intValue <<= v.intValue;
+    return ret;
+}
+
+MathLib::value MathLib::value::shiftRight(const MathLib::value &v) const
+{
+    if (!isInt() || !v.isInt())
+        throw InternalError(nullptr, "Shift operand is not integer");
+    MathLib::value ret(*this);
+    if (v.intValue >= MathLib::bigint_bits) {
+        return ret;
+    }
+    ret.intValue >>= v.intValue;
+    return ret;
+}
+
 
 MathLib::biguint MathLib::toULongNumber(const std::string & str)
 {
@@ -313,6 +344,143 @@ MathLib::biguint MathLib::toULongNumber(const std::string & str)
     return ret;
 }
 
+static unsigned int encodeMultiChar(const std::string& str)
+{
+    unsigned int retval = 0;
+    for (std::string::const_iterator it=str.begin(); it!=str.end(); ++it) {
+        retval = (retval << 8) | *it;
+    }
+    return retval;
+}
+
+static bool isoctal(int c)
+{
+    return c>='0' && c<='7';
+}
+
+MathLib::bigint MathLib::characterLiteralToLongNumber(const std::string& str)
+{
+    if (str.empty())
+        return 0; // <- only possible in unit testing
+
+    // '\xF6'
+    if (str.size() == 4 && str.compare(0,2,"\\x")==0 && std::isxdigit(str[2]) && std::isxdigit(str[3])) {
+        return std::strtoul(str.substr(2).c_str(), nullptr, 16);
+    }
+
+    // '\123'
+    if (str.size() == 4 && str[0] == '\\' && isoctal(str[1]) && isoctal(str[2]) && isoctal(str[3])) {
+        return (char)std::strtoul(str.substr(1).c_str(), NULL, 8);
+    }
+
+    // C99 6.4.4.4
+    // The value of an integer character constant containing more than one character (e.g., 'ab'),
+    // or containing a character or escape sequence that does not map to a single-byte execution character,
+    // is implementation-defined.
+    // clang and gcc seem to use the following encoding: 'AB' as (('A' << 8) | 'B')
+    const std::string& normStr = normalizeCharacterLiteral(str);
+    return encodeMultiChar(normStr);
+}
+
+std::string MathLib::normalizeCharacterLiteral(const std::string& iLiteral)
+{
+    std::string normalizedLiteral;
+    const std::string::size_type iLiteralLen = iLiteral.size();
+    for (std::string::size_type idx = 0; idx < iLiteralLen ; ++idx) {
+        if (iLiteral[idx] != '\\') {
+            normalizedLiteral.push_back(iLiteral[idx]);
+            continue;
+        }
+        ++idx;
+        if (idx == iLiteralLen) {
+            throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
+        }
+        switch (iLiteral[idx]) {
+        case 'x':
+            // Hexa-decimal number: skip \x and interpret the next two characters
+        {
+            if (++idx == iLiteralLen)
+                throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
+            std::string tempBuf;
+            tempBuf.push_back(iLiteral[idx]);
+            if (++idx != iLiteralLen)
+                tempBuf.push_back(iLiteral[idx]);
+            normalizedLiteral.push_back(static_cast<char>(MathLib::toULongNumber("0x" + tempBuf)));
+            continue;
+        }
+        case 'u':
+        case 'U':
+            // Unicode string; just skip the \u or \U
+            if (idx + 1 == iLiteralLen)
+                throw InternalError(nullptr, "Internal Error. MathLib::characterLiteralToLongNumber: Unhandled char constant '" + iLiteral + "'.");
+            continue;
+        }
+        // Single digit octal number
+        if (1 == iLiteralLen - idx) {
+            switch (iLiteral[idx]) {
+            case '0':
+            case '1':
+            case '2':
+            case '3':
+            case '4':
+            case '5':
+            case '6':
+            case '7':
+                normalizedLiteral.push_back(iLiteral[idx]-'0');
+                break;
+            case 'a':
+                normalizedLiteral.push_back('\a');
+                break;
+            case 'b':
+                normalizedLiteral.push_back('\b');
+                break;
+            case 'e':
+                normalizedLiteral.push_back(0x1B); // clang, gcc, tcc interpnormalizedLiteral this as 0x1B - escape character
+                break;
+            case 'f':
+                normalizedLiteral.push_back('\f');
+                break;
+            case 'n':
+                normalizedLiteral.push_back('\n');
+                break;
+            case 'r':
+                normalizedLiteral.push_back('\r');
+                break;
+            case 't':
+                normalizedLiteral.push_back('\t');
+                break;
+            case 'v':
+                normalizedLiteral.push_back('\v');
+                break;
+            case '\\':
+            case '\?':
+            case '\'':
+            case '\"':
+                normalizedLiteral.push_back(iLiteral[idx]);
+                break;
+            default:
+                throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
+            }
+            continue;
+        }
+        // 2-3 digit octal number
+        if (!MathLib::isOctalDigit(iLiteral[idx]))
+            throw InternalError(nullptr, "Internal Error. MathLib::normalizeCharacterLiteral: Unhandled char constant '" + iLiteral + "'.");
+        std::string tempBuf;
+        tempBuf.push_back(iLiteral[idx]);
+        ++idx;
+        if (MathLib::isOctalDigit(iLiteral[idx])) {
+            tempBuf.push_back(iLiteral[idx]);
+            ++idx;
+            if (MathLib::isOctalDigit(iLiteral[idx])) {
+                tempBuf.push_back(iLiteral[idx]);
+            }
+        }
+        normalizedLiteral.push_back(static_cast<char>(MathLib::toLongNumber("0" + tempBuf)));
+    }
+    return normalizedLiteral;
+}
+
 MathLib::bigint MathLib::toLongNumber(const std::string & str)
 {
     // hexadecimal numbers:
@@ -363,7 +531,18 @@ MathLib::bigint MathLib::toLongNumber(const std::string & str)
             return static_cast<bigint>(doubleval);
     }
 
-    bigint ret = 0;
+    if (str[0] == '\'' && str.size() >= 3U && endsWith(str,'\'')) {
+        return characterLiteralToLongNumber(str.substr(1,str.size()-2));
+    }
+
+    if (str[0] == '-') {
+        bigint ret = 0;
+        std::istringstream istr(str);
+        istr >> ret;
+        return ret;
+    }
+
+    biguint ret = 0;
     std::istringstream istr(str);
     istr >> ret;
     return ret;
@@ -372,15 +551,17 @@ MathLib::bigint MathLib::toLongNumber(const std::string & str)
 
 double MathLib::toDoubleNumber(const std::string &str)
 {
+    if (str[0] == '\'' && str.size() >= 3U && endsWith(str,'\''))
+        return characterLiteralToLongNumber(str.substr(1,str.size()-2));
     if (isIntHex(str))
         return static_cast<double>(toLongNumber(str));
     // nullcheck
-    else if (isNullValue(str))
+    if (isNullValue(str))
         return 0.0;
 #ifdef __clang__
-    else if (isFloat(str)) // Workaround libc++ bug at http://llvm.org/bugs/show_bug.cgi?id=17782
+    if (isFloat(str)) // Workaround libc++ bug at http://llvm.org/bugs/show_bug.cgi?id=17782
         // TODO : handle locale
-        return std::strtod(str.c_str(), 0);
+        return std::strtod(str.c_str(), nullptr);
 #endif
     // otherwise, convert to double
     std::istringstream istr(str);
@@ -402,19 +583,19 @@ template<> std::string MathLib::toString(double value)
     return result.str();
 }
 
-bool MathLib::isFloat(const std::string &s)
+bool MathLib::isFloat(const std::string &str)
 {
-    return isDecimalFloat(s) || isFloatHex(s);
+    return isDecimalFloat(str) || isFloatHex(str);
 }
 
-bool MathLib::isDecimalFloat(const std::string &s)
+bool MathLib::isDecimalFloat(const std::string &str)
 {
-    if (s.empty())
+    if (str.empty())
         return false;
     enum State {
         START, BASE_PLUSMINUS, BASE_DIGITS1, LEADING_DECIMAL, TRAILING_DECIMAL, BASE_DIGITS2, E, MANTISSA_PLUSMINUS, MANTISSA_DIGITS, SUFFIX_F, SUFFIX_L
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it=='+' || *it=='-')
@@ -505,18 +686,18 @@ bool MathLib::isDecimalFloat(const std::string &s)
     return (state==BASE_DIGITS2 || state==MANTISSA_DIGITS || state==TRAILING_DECIMAL || state==SUFFIX_F || state==SUFFIX_L);
 }
 
-bool MathLib::isNegative(const std::string &s)
+bool MathLib::isNegative(const std::string &str)
 {
-    if (s.empty())
+    if (str.empty())
         return false;
-    return (s[0] == '-');
+    return (str[0] == '-');
 }
 
-bool MathLib::isPositive(const std::string &s)
+bool MathLib::isPositive(const std::string &str)
 {
-    if (s.empty())
+    if (str.empty())
         return false;
-    return !MathLib::isNegative(s);
+    return !MathLib::isNegative(str);
 }
 
 /*! \brief Does the string represent an octal number?
@@ -525,15 +706,15 @@ bool MathLib::isPositive(const std::string &s)
  * Additional information can be found here:
  * http://gcc.gnu.org/onlinedocs/gcc/Binary-constants.html
  *
- * \param[in] s The string to check. In case the string is empty, the function returns false.
+ * \param str The string to check. In case the string is empty, the function returns false.
  * \return Return true in case a octal number is provided and false otherwise.
  **/
-bool MathLib::isOct(const std::string& s)
+bool MathLib::isOct(const std::string& str)
 {
     enum Status {
         START, PLUSMINUS, OCTAL_PREFIX, DIGITS
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it == '+' || *it == '-')
@@ -559,19 +740,19 @@ bool MathLib::isOct(const std::string& s)
             if (isOctalDigit(static_cast<unsigned char>(*it)))
                 state = DIGITS;
             else
-                return isValidIntegerSuffix(it,s.end());
+                return isValidIntegerSuffix(it,str.end());
             break;
         }
     }
     return state == DIGITS;
 }
 
-bool MathLib::isIntHex(const std::string& s)
+bool MathLib::isIntHex(const std::string& str)
 {
     enum Status {
         START, PLUSMINUS, HEX_PREFIX, DIGIT, DIGITS
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it == '+' || *it == '-')
@@ -603,19 +784,19 @@ bool MathLib::isIntHex(const std::string& s)
             if (isxdigit(static_cast<unsigned char>(*it)))
                 state = DIGITS;
             else
-                return isValidIntegerSuffix(it,s.end());
+                return isValidIntegerSuffix(it,str.end());
             break;
         }
     }
     return state == DIGITS;
 }
 
-bool MathLib::isFloatHex(const std::string& s)
+bool MathLib::isFloatHex(const std::string& str)
 {
     enum Status {
         START, PLUSMINUS, HEX_PREFIX, WHOLE_NUMBER_DIGIT, WHOLE_NUMBER_DIGITS, FRACTION, EXPONENT_DIGIT, EXPONENT_DIGITS
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it == '+' || *it == '-')
@@ -678,9 +859,14 @@ bool MathLib::isFloatHex(const std::string& s)
     return state==EXPONENT_DIGITS;
 }
 
+bool MathLib::isValidIntegerSuffix(const std::string& str)
+{
+    return isValidIntegerSuffix(str.begin(), str.end());
+}
+
 bool MathLib::isValidIntegerSuffix(std::string::const_iterator it, std::string::const_iterator end)
 {
-    enum {START, SUFFIX_U, SUFFIX_UL, SUFFIX_ULL, SUFFIX_L, SUFFIX_LU, SUFFIX_LL, SUFFIX_LLU, SUFFIX_I, SUFFIX_I6, SUFFIX_I64} state = START;
+    enum {START, SUFFIX_U, SUFFIX_UL, SUFFIX_ULL, SUFFIX_L, SUFFIX_LU, SUFFIX_LL, SUFFIX_LLU, SUFFIX_I, SUFFIX_I6, SUFFIX_I64, SUFFIX_UI, SUFFIX_UI6, SUFFIX_UI64} state = START;
     for (; it != end; ++it) {
         switch (state) {
         case START:
@@ -696,6 +882,8 @@ bool MathLib::isValidIntegerSuffix(std::string::const_iterator it, std::string::
         case SUFFIX_U:
             if (*it == 'l' || *it == 'L')
                 state = SUFFIX_UL; // UL
+            else if (*it == 'i')
+                state = SUFFIX_UI;
             else
                 return false;
             break;
@@ -733,6 +921,18 @@ bool MathLib::isValidIntegerSuffix(std::string::const_iterator it, std::string::
             else
                 return false;
             break;
+        case SUFFIX_UI:
+            if (*it == '6')
+                state = SUFFIX_UI6;
+            else
+                return false;
+            break;
+        case SUFFIX_UI6:
+            if (*it == '4')
+                state = SUFFIX_UI64;
+            else
+                return false;
+            break;
         default:
             return false;
         }
@@ -744,7 +944,8 @@ bool MathLib::isValidIntegerSuffix(std::string::const_iterator it, std::string::
             (state == SUFFIX_LL)  ||
             (state == SUFFIX_ULL) ||
             (state == SUFFIX_LLU) ||
-            (state == SUFFIX_I64));
+            (state == SUFFIX_I64) ||
+            (state == SUFFIX_UI64));
 }
 
 /*! \brief Does the string represent a binary number?
@@ -753,15 +954,15 @@ bool MathLib::isValidIntegerSuffix(std::string::const_iterator it, std::string::
  * Additional information can be found here:
  * http://gcc.gnu.org/onlinedocs/gcc/Binary-constants.html
  *
- * \param[in] s The string to check. In case the string is empty, the function returns false.
+ * \param str The string to check. In case the string is empty, the function returns false.
  * \return Return true in case a binary number is provided and false otherwise.
  **/
-bool MathLib::isBin(const std::string& s)
+bool MathLib::isBin(const std::string& str)
 {
     enum Status {
         START, PLUSMINUS, GNU_BIN_PREFIX, DIGIT, DIGITS
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it == '+' || *it == '-')
@@ -793,19 +994,19 @@ bool MathLib::isBin(const std::string& s)
             if (*it == '0' || *it == '1')
                 state = DIGITS;
             else
-                return isValidIntegerSuffix(it,s.end());
+                return isValidIntegerSuffix(it,str.end());
             break;
         }
     }
     return state == DIGITS;
 }
 
-bool MathLib::isDec(const std::string & s)
+bool MathLib::isDec(const std::string & str)
 {
     enum Status {
         START, PLUSMINUS, DIGIT
     } state = START;
-    for (std::string::const_iterator it = s.begin(); it != s.end(); ++it) {
+    for (std::string::const_iterator it = str.begin(); it != str.end(); ++it) {
         switch (state) {
         case START:
             if (*it == '+' || *it == '-')
@@ -825,45 +1026,53 @@ bool MathLib::isDec(const std::string & s)
             if (isdigit(static_cast<unsigned char>(*it)))
                 state = DIGIT;
             else
-                return isValidIntegerSuffix(it,s.end());
+                return isValidIntegerSuffix(it,str.end());
             break;
         }
     }
     return state == DIGIT;
 }
 
-bool MathLib::isInt(const std::string & s)
+bool MathLib::isInt(const std::string & str)
 {
-    return isDec(s) || isIntHex(s) || isOct(s) || isBin(s);
+    return isDec(str) || isIntHex(str) || isOct(str) || isBin(str);
+}
+
+std::string MathLib::getSuffix(const std::string& value)
+{
+    if (value.size() > 3 && value[value.size() - 3] == 'i' && value[value.size() - 2] == '6' && value[value.size() - 1] == '4') {
+        if (value[value.size() - 4] == 'u')
+            return "ULL";
+        return "LL";
+    }
+    bool isUnsigned = false;
+    unsigned int longState = 0;
+    for (std::size_t i = 1U; i < value.size(); ++i) {
+        const char c = value[value.size() - i];
+        if (c == 'u' || c == 'U')
+            isUnsigned = true;
+        else if (c == 'L' || c == 'l')
+            longState++;
+        else break;
+    }
+    if (longState == 0)
+        return isUnsigned ? "U" : "";
+    if (longState == 1)
+        return isUnsigned ? "UL" : "L";
+    if (longState == 2)
+        return isUnsigned ? "ULL" : "LL";
+    else return "";
 }
 
 static std::string intsuffix(const std::string & first, const std::string & second)
 {
-    std::string suffix1, suffix2;
-    for (std::size_t i = 1U; i < first.size(); ++i) {
-        char c = first[first.size() - i];
-        if (c == 'l' || c == 'u')
-            c = c - 'a' + 'A';
-        if (c != 'L' && c != 'U')
-            break;
-        suffix1 = c + suffix1;
-    }
-    for (std::size_t i = 1U; i < second.size(); ++i) {
-        char c = second[second.size() - i];
-        if (c == 'l' || c == 'u')
-            c = c - 'a' + 'A';
-        if (c != 'L' && c != 'U')
-            break;
-        suffix2 = c + suffix2;
-    }
-
-    if (suffix1 == "ULL" || suffix2 == "ULL"
-        || suffix1 == "LLU" || suffix2 == "LLU")
+    const std::string suffix1 = MathLib::getSuffix(first);
+    const std::string suffix2 = MathLib::getSuffix(second);
+    if (suffix1 == "ULL" || suffix2 == "ULL")
         return "ULL";
     if (suffix1 == "LL" || suffix2 == "LL")
         return "LL";
-    if (suffix1 == "UL" || suffix2 == "UL"
-        || suffix1 == "LU" || suffix2 == "LU")
+    if (suffix1 == "UL" || suffix2 == "UL")
         return "UL";
     if (suffix1 == "L" || suffix2 == "L")
         return "L";
@@ -934,7 +1143,7 @@ std::string MathLib::incdec(const std::string & var, const std::string & op)
         return MathLib::subtract(var, "1");
 #endif
 
-    throw InternalError(0, std::string("Unexpected operation '") + op + "' in MathLib::incdec(). Please report this to Cppcheck developers.");
+    throw InternalError(nullptr, std::string("Unexpected operation '") + op + "' in MathLib::incdec(). Please report this to Cppcheck developers.");
 }
 
 std::string MathLib::divide(const std::string &first, const std::string &second)
@@ -946,9 +1155,9 @@ std::string MathLib::divide(const std::string &first, const std::string &second)
         const bigint a = toLongNumber(first);
         const bigint b = toLongNumber(second);
         if (b == 0)
-            throw InternalError(0, "Internal Error: Division by zero");
+            throw InternalError(nullptr, "Internal Error: Division by zero");
         if (a == std::numeric_limits<bigint>::min() && std::abs(b)<=1)
-            throw InternalError(0, "Internal Error: Division overflow");
+            throw InternalError(nullptr, "Internal Error: Division overflow");
         return toString(toLongNumber(first) / b) + intsuffix(first, second);
     } else if (isNullValue(second)) {
         if (isNullValue(first))
@@ -979,7 +1188,7 @@ std::string MathLib::mod(const std::string &first, const std::string &second)
     if (MathLib::isInt(first) && MathLib::isInt(second)) {
         const bigint b = toLongNumber(second);
         if (b == 0)
-            throw InternalError(0, "Internal Error: Division by zero");
+            throw InternalError(nullptr, "Internal Error: Division by zero");
         return toString(toLongNumber(first) % b) + intsuffix(first, second);
     }
     return toString(std::fmod(toDoubleNumber(first),toDoubleNumber(second)));
@@ -1014,7 +1223,7 @@ std::string MathLib::calculate(const std::string &first, const std::string &seco
         return MathLib::toString(MathLib::toLongNumber(first) ^ MathLib::toLongNumber(second)) + intsuffix(first,second);
 
     default:
-        throw InternalError(0, std::string("Unexpected action '") + action + "' in MathLib::calculate(). Please report this to Cppcheck developers.");
+        throw InternalError(nullptr, std::string("Unexpected action '") + action + "' in MathLib::calculate(). Please report this to Cppcheck developers.");
     }
 }
 
@@ -1100,6 +1309,45 @@ bool MathLib::isOctalDigit(char c)
     return (c >= '0' && c <= '7');
 }
 
+bool MathLib::isDigitSeparator(const std::string& iCode, std::string::size_type iPos)
+{
+    if (iPos == 0 || iPos >= iCode.size() || iCode[iPos] != '\'')
+        return false;
+    std::string::size_type i = iPos - 1;
+    while (std::isxdigit(iCode[i])) {
+        if (i == 0)
+            return true; // Only xdigits before '
+        --i;
+    }
+    if (i == iPos - 1) { // No xdigit before '
+        return false;
+    } else {
+        switch (iCode[i]) {
+        case ' ':
+        case '.':
+        case ',':
+        case 'x':
+        case '(':
+        case '{':
+        case '+':
+        case '-':
+        case '*':
+        case '%':
+        case '/':
+        case '&':
+        case '|':
+        case '^':
+        case '~':
+        case '=':
+            return true;
+        case '\'':
+            return isDigitSeparator(iCode, i);
+        default:
+            return false;
+        }
+    }
+}
+
 MathLib::value operator+(const MathLib::value &v1, const MathLib::value &v2)
 {
     return MathLib::value::calc('+',v1,v2);
@@ -1139,3 +1387,13 @@ MathLib::value operator^(const MathLib::value &v1, const MathLib::value &v2)
 {
     return MathLib::value::calc('^',v1,v2);
 }
+
+MathLib::value operator<<(const MathLib::value &v1, const MathLib::value &v2)
+{
+    return v1.shiftLeft(v2);
+}
+
+MathLib::value operator>>(const MathLib::value &v1, const MathLib::value &v2)
+{
+    return v1.shiftRight(v2);
+}
