@@ -448,9 +448,9 @@ std::set<std::string> TemplateSimplifier::expandSpecialized(Token *tokens)
 /// TODO: This is copy pasted from Tokenizer. We should reuse this code.
 namespace {
     struct ScopeInfo2 {
-        ScopeInfo2(const std::string &name_, const Token *classEnd_) : name(name_), classEnd(classEnd_) {}
+        ScopeInfo2(const std::string &name_, const Token *bodyEnd_) : name(name_), bodyEnd(bodyEnd_) {}
         const std::string name;
-        const Token * const classEnd;
+        const Token * const bodyEnd;
     };
 }
 static std::string getScopeName(const std::list<ScopeInfo2> &scopeInfo)
@@ -468,7 +468,7 @@ static std::string getFullName(const std::list<ScopeInfo2> &scopeInfo, const std
 
 static void setScopeInfo(const Token *tok, std::list<ScopeInfo2> *scopeInfo)
 {
-    while (tok->str() == "}" && !scopeInfo->empty() && tok == scopeInfo->back().classEnd)
+    while (tok->str() == "}" && !scopeInfo->empty() && tok == scopeInfo->back().bodyEnd)
         scopeInfo->pop_back();
     if (!Token::Match(tok, "namespace|class|struct %name% {|:|::"))
         return;
@@ -483,7 +483,7 @@ static void setScopeInfo(const Token *tok, std::list<ScopeInfo2> *scopeInfo)
         // ...
     }
     if (tok && tok->str() == "{") {
-        scopeInfo->push_back(ScopeInfo2(classname,tok->link()));
+        scopeInfo->emplace_back(classname,tok->link());
     }
 }
 
@@ -492,14 +492,17 @@ std::list<TemplateSimplifier::TokenAndName> TemplateSimplifier::getTemplateDecla
     std::list<ScopeInfo2> scopeInfo;
     std::list<TokenAndName> declarations;
     for (Token *tok = tokens; tok; tok = tok->next()) {
-        setScopeInfo(tok, &scopeInfo);
+        if (Token::Match(tok, "}|namespace|class|struct")) {
+            setScopeInfo(tok, &scopeInfo);
+            continue;
+        }
         if (!Token::simpleMatch(tok, "template <"))
             continue;
         // Some syntax checks, see #6865
         if (!tok->tokAt(2))
             syntaxError(tok->next());
         if (tok->strAt(2)=="typename" &&
-            !Token::Match(tok->tokAt(3), "%name%|.|,|>"))
+            !Token::Match(tok->tokAt(3), "%name%|.|,|=|>"))
             syntaxError(tok->next());
         codeWithTemplates = true;
         Token *parmEnd = tok->next()->findClosingBracket();
@@ -513,9 +516,9 @@ std::list<TemplateSimplifier::TokenAndName> TemplateSimplifier::getTemplateDecla
                 break;
             // Implementation => add to "templates"
             else if (tok2->str() == "{") {
-                int namepos = getTemplateNamePosition(parmEnd);
+                const int namepos = getTemplateNamePosition(parmEnd);
                 if (namepos > 0)
-                    declarations.push_back(TokenAndName(tok, getScopeName(scopeInfo), getFullName(scopeInfo, parmEnd->strAt(namepos))));
+                    declarations.emplace_back(tok, getScopeName(scopeInfo), getFullName(scopeInfo, parmEnd->strAt(namepos)));
                 break;
             }
         }
@@ -530,7 +533,10 @@ std::list<TemplateSimplifier::TokenAndName> TemplateSimplifier::getTemplateInsta
     std::list<ScopeInfo2> scopeList;
 
     for (Token *tok = tokens; tok; tok = tok->next()) {
-        setScopeInfo(tok, &scopeList);
+        if (Token::Match(tok, "}|namespace|class|struct")) {
+            setScopeInfo(tok, &scopeList);
+            continue;
+        }
         // template definition.. skip it
         if (Token::simpleMatch(tok, "template <")) {
             tok = tok->next()->findClosingBracket();
@@ -563,7 +569,7 @@ std::list<TemplateSimplifier::TokenAndName> TemplateSimplifier::getTemplateInsta
             for (; tok2 && tok2 != tok; tok2 = tok2->previous()) {
                 if (Token::Match(tok2, ", %name% <") &&
                     TemplateSimplifier::templateParameters(tok2->tokAt(2))) {
-                    instantiations.push_back(TokenAndName(tok2->next(), getScopeName(scopeList), getFullName(scopeList, tok2->strAt(1))));
+                    instantiations.emplace_back(tok2->next(), getScopeName(scopeList), getFullName(scopeList, tok2->strAt(1)));
                 }
             }
 
@@ -574,11 +580,11 @@ std::list<TemplateSimplifier::TokenAndName> TemplateSimplifier::getTemplateInsta
                     const std::string fullName = scopeName + (scopeName.empty()?"":" :: ") + tok->str();
                     const std::list<TokenAndName>::const_iterator it = std::find_if(declarations.begin(), declarations.end(), FindName(fullName));
                     if (it != declarations.end()) {
-                        instantiations.push_back(TokenAndName(tok, getScopeName(scopeList), fullName));
+                        instantiations.emplace_back(tok, getScopeName(scopeList), fullName);
                         break;
                     } else {
                         if (scopeName.empty()) {
-                            instantiations.push_back(TokenAndName(tok, getScopeName(scopeList), scopeName1 + (scopeName1.empty()?"":" :: ") + tok->str()));
+                            instantiations.emplace_back(tok, getScopeName(scopeList), scopeName1 + (scopeName1.empty()?"":" :: ") + tok->str());
                             break;
                         }
                         const std::string::size_type pos = scopeName.rfind(" :: ");
@@ -781,7 +787,7 @@ void TemplateSimplifier::simplifyTemplateAliases(std::list<TemplateSimplifier::T
                     tok2 = tok2->next();
                 }
 
-                args.push_back(std::pair<Token *, Token *>(start, tok2));
+                args.emplace_back(start, tok2);
                 if (tok2 && tok2->str() == ",") {
                     tok2 = tok2->next();
                 } else {
@@ -813,7 +819,7 @@ void TemplateSimplifier::simplifyTemplateAliases(std::list<TemplateSimplifier::T
                                 templateInstantiations->end(),
                                 FindToken(tok1));
                         if (it != templateInstantiations->end())
-                            templateInstantiations->push_back(TokenAndName(tok2, it->scope, it->name));
+                            templateInstantiations->emplace_back(tok2, it->scope, it->name);
                     }
                     continue;
                 }
@@ -949,7 +955,10 @@ void TemplateSimplifier::expandTemplate(
     const Token *endOfTemplateDefinition = nullptr;
     const Token * const templateDeclarationNameToken = templateDeclarationToken->tokAt(getTemplateNamePosition(templateDeclarationToken));
     for (const Token *tok3 = tokenlist.front(); tok3; tok3 = tok3 ? tok3->next() : nullptr) {
-        setScopeInfo(const_cast<Token *>(tok3), &scopeInfo);
+        if (Token::Match(tok3, "}|namespace|class|struct")) {
+            setScopeInfo(tok3, &scopeInfo);
+            continue;
+        }
         if (inTemplateDefinition) {
             if (!endOfTemplateDefinition && tok3->str() == "{")
                 endOfTemplateDefinition = tok3->link();
@@ -1075,7 +1084,7 @@ void TemplateSimplifier::expandTemplate(
                 std::string name = tok3->str();
                 for (const Token *prev = tok3->tokAt(-2); Token::Match(prev, "%name% ::"); prev = prev->tokAt(-2))
                     name = prev->str() + " :: " + name;
-                templateInstantiations.push_back(TokenAndName(tokenlist.back(), getScopeName(scopeInfo), getFullName(scopeInfo, name)));
+                templateInstantiations.emplace_back(tokenlist.back(), getScopeName(scopeInfo), getFullName(scopeInfo, name));
             }
 
             // link() newly tokens manually
@@ -1150,41 +1159,44 @@ bool TemplateSimplifier::simplifyNumericCalculations(Token *tok)
 {
     bool ret = false;
     // (1-2)
-    while (tok->tokAt(4) && tok->next()->isNumber() && tok->tokAt(3)->isNumber()) { // %any% %num% %any% %num% %any%
-        const Token* op = tok->tokAt(2);
-        const Token* after = tok->tokAt(4);
-        if (Token::Match(tok, "* %num% /") && (op->strAt(1) != "0") && tok->next()->str() == MathLib::multiply(op->strAt(1), MathLib::divide(tok->next()->str(), op->strAt(1)))) {
+    while (tok->tokAt(3) && tok->isNumber() && tok->tokAt(2)->isNumber()) { // %any% %num% %any% %num% %any%
+        const Token *before = tok->previous();
+        if (!before)
+            break;
+        const Token* op = tok->next();
+        const Token* after = tok->tokAt(3);
+        const std::string &num1 = op->previous()->str();
+        const std::string &num2 = op->next()->str();
+        if (Token::Match(before, "* %num% /") && (num2 != "0") && num1 == MathLib::multiply(num2, MathLib::divide(num1, num2))) {
             // Division where result is a whole number
-        } else if (!((op->str() == "*" && (isLowerThanMulDiv(tok) || tok->str() == "*") && isLowerEqualThanMulDiv(after)) || // associative
-                     (Token::Match(op, "[/%]") && isLowerThanMulDiv(tok) && isLowerEqualThanMulDiv(after)) || // NOT associative
-                     (Token::Match(op, "[+-]") && isLowerThanMulDiv(tok) && isLowerThanMulDiv(after)) || // Only partially (+) associative, but handled later
-                     (Token::Match(op, ">>|<<") && isLowerThanShift(tok) && isLowerThanPlusMinus(after)) || // NOT associative
-                     (op->str() == "&" && isLowerThanShift(tok) && isLowerThanShift(after)) || // associative
-                     (op->str() == "^" && isLowerThanAnd(tok) && isLowerThanAnd(after)) || // associative
-                     (op->str() == "|" && isLowerThanXor(tok) && isLowerThanXor(after)) || // associative
-                     (op->str() == "&&" && isLowerThanOr(tok) && isLowerThanOr(after)) ||
-                     (op->str() == "||" && isLowerThanLogicalAnd(tok) && isLowerThanLogicalAnd(after))))
+        } else if (!((op->str() == "*" && (isLowerThanMulDiv(before) || before->str() == "*") && isLowerEqualThanMulDiv(after)) || // associative
+                     (Token::Match(op, "[/%]") && isLowerThanMulDiv(before) && isLowerEqualThanMulDiv(after)) || // NOT associative
+                     (Token::Match(op, "[+-]") && isLowerThanMulDiv(before) && isLowerThanMulDiv(after)) || // Only partially (+) associative, but handled later
+                     (Token::Match(op, ">>|<<") && isLowerThanShift(before) && isLowerThanPlusMinus(after)) || // NOT associative
+                     (op->str() == "&" && isLowerThanShift(before) && isLowerThanShift(after)) || // associative
+                     (op->str() == "^" && isLowerThanAnd(before) && isLowerThanAnd(after)) || // associative
+                     (op->str() == "|" && isLowerThanXor(before) && isLowerThanXor(after)) || // associative
+                     (op->str() == "&&" && isLowerThanOr(before) && isLowerThanOr(after)) ||
+                     (op->str() == "||" && isLowerThanLogicalAnd(before) && isLowerThanLogicalAnd(after))))
             break;
-
-        tok = tok->next();
 
         // Don't simplify "%num% / 0"
         if (Token::Match(op, "[/%] 0"))
-            continue;
+            break;
 
         // Integer operations
         if (Token::Match(op, ">>|<<|&|^|%or%")) {
             // Don't simplify if operand is negative, shifting with negative
             // operand is UB. Bitmasking with negative operand is implementation
             // defined behaviour.
-            if (MathLib::isNegative(tok->str()) || MathLib::isNegative(tok->strAt(2)))
-                continue;
+            if (MathLib::isNegative(num1) || MathLib::isNegative(num2))
+                break;
 
-            const MathLib::value v1(tok->str());
-            const MathLib::value v2(tok->strAt(2));
+            const MathLib::value v1(num1);
+            const MathLib::value v2(num2);
 
             if (!v1.isInt() || !v2.isInt())
-                continue;
+                break;
 
             switch (op->str()[0]) {
             case '<':
@@ -1207,19 +1219,19 @@ bool TemplateSimplifier::simplifyNumericCalculations(Token *tok)
 
         // Logical operations
         else if (Token::Match(op, "%oror%|&&")) {
-            bool op1 = !MathLib::isNullValue(tok->str());
-            bool op2 = !MathLib::isNullValue(tok->strAt(2));
-            bool result = (op->str() == "||") ? (op1 || op2) : (op1 && op2);
+            const bool op1 = !MathLib::isNullValue(num1);
+            const bool op2 = !MathLib::isNullValue(num2);
+            const bool result = (op->str() == "||") ? (op1 || op2) : (op1 && op2);
             tok->str(result ? "1" : "0");
         }
 
         else if (Token::Match(tok->previous(), "- %num% - %num%"))
-            tok->str(MathLib::add(tok->str(), tok->strAt(2)));
+            tok->str(MathLib::add(num1, num2));
         else if (Token::Match(tok->previous(), "- %num% + %num%"))
-            tok->str(MathLib::subtract(tok->str(), tok->strAt(2)));
+            tok->str(MathLib::subtract(num1, num2));
         else {
             try {
-                tok->str(MathLib::calculate(tok->str(), tok->strAt(2), op->str()[0]));
+                tok->str(MathLib::calculate(num1, num2, op->str()[0]));
             } catch (InternalError &e) {
                 e.token = tok;
                 throw;
@@ -1227,15 +1239,10 @@ bool TemplateSimplifier::simplifyNumericCalculations(Token *tok)
         }
 
         tok->deleteNext(2);
-        tok = tok->previous();
 
         ret = true;
     }
 
-    if (Token::Match(tok, "%oror%|&& %num% %oror%|&&|,|)") || Token::Match(tok, "[(,] %num% %oror%|&&")) {
-        tok->next()->str(MathLib::isNullValue(tok->next()->str()) ? "0" : "1");
-    }
-
     return ret;
 }
 
@@ -1243,12 +1250,8 @@ bool TemplateSimplifier::simplifyNumericCalculations(Token *tok)
 // should be moved away.
 bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
 {
-    bool ret = false, goback = false;
+    bool ret = false;
     for (Token *tok = _tokens; tok; tok = tok->next()) {
-        if (goback) {
-            tok = tok->previous();
-            goback = false;
-        }
         // Remove parentheses around variable..
         // keep parentheses here: dynamic_cast<Fred *>(p);
         // keep parentheses here: A operator * (int);
@@ -1274,12 +1277,21 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
         }
 
         if (tok->isNumber()) {
+            if (simplifyNumericCalculations(tok)) {
+                ret = true;
+                Token *prev = tok->tokAt(-2);
+                while (prev && simplifyNumericCalculations(prev)) {
+                    tok = prev;
+                    prev = prev->tokAt(-2);
+                }
+            }
+
             // Remove redundant conditions (0&&x) (1||x)
             if (Token::Match(tok->previous(), "[(=,] 0 &&") ||
                 Token::Match(tok->previous(), "[(=,] 1 %oror%")) {
                 unsigned int par = 0;
                 const Token *tok2 = tok;
-                bool andAnd = (tok->next()->str() == "&&");
+                const bool andAnd = (tok->next()->str() == "&&");
                 for (; tok2; tok2 = tok2->next()) {
                     if (tok2->str() == "(" || tok2->str() == "[")
                         ++par;
@@ -1405,26 +1417,10 @@ bool TemplateSimplifier::simplifyCalculations(Token *_tokens)
                     tok->str(result);
                     tok->deleteNext(2);
                     ret = true;
-                    goback = true;
+                    tok = tok->previous();
                 }
             }
         }
-        // Division where result is a whole number
-        else if (Token::Match(tok->previous(), "* %num% /") &&
-                 tok->str() == MathLib::multiply(tok->strAt(2), MathLib::divide(tok->str(), tok->strAt(2)))) {
-            tok->deleteNext(2);
-        }
-
-        else if (simplifyNumericCalculations(tok)) {
-            ret = true;
-            while (Token::Match(tok->tokAt(-2), "%cop%|,|( %num% %cop% %num% %cop%|,|)")) {
-                Token *before = tok->tokAt(-2);
-                if (simplifyNumericCalculations(before))
-                    tok = before;
-                else
-                    break;
-            }
-        }
     }
     return ret;
 }
@@ -1510,7 +1506,7 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
     if (namepos == -1) {
         // debug message that we bail out..
         if (printDebug && errorlogger) {
-            std::list<const Token *> callstack(1, tok);
+            const std::list<const Token *> callstack(1, tok);
             errorlogger->reportErr(ErrorLogger::ErrorMessage(callstack, &tokenlist, Severity::debug, "debug", "simplifyTemplates: bailing out", false));
         }
         return false;
@@ -1524,7 +1520,7 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
 
     bool instantiated = false;
 
-    for (std::list<TokenAndName>::const_iterator iter2 = templateInstantiations.begin(); iter2 != templateInstantiations.end(); ++iter2) {
+    for (const TokenAndName &instantiation : templateInstantiations) {
         if (numberOfTemplateInstantiations != templateInstantiations.size()) {
             numberOfTemplateInstantiations = templateInstantiations.size();
             simplifyCalculations(tokenlist.front());
@@ -1536,16 +1532,16 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
         }
 
         // already simplified
-        if (!Token::Match(iter2->token, "%name% <"))
+        if (!Token::Match(instantiation.token, "%name% <"))
             continue;
 
-        if (iter2->name != templateDeclaration.name)
+        if (instantiation.name != templateDeclaration.name)
             continue;
 
-        if (!matchSpecialization(tok->tokAt(namepos), iter2->token, specializations))
+        if (!matchSpecialization(tok->tokAt(namepos), instantiation.token, specializations))
             continue;
 
-        Token * const tok2 = iter2->token;
+        Token * const tok2 = instantiation.token;
         if (errorlogger && !tokenlist.getFiles().empty())
             errorlogger->reportProgress(tokenlist.getFiles()[0], "TemplateSimplifier::simplifyTemplateInstantiations()", tok2->progressValue());
 #ifdef MAXTIME
@@ -1580,7 +1576,7 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
                 typeForNewName.clear();
                 break;
             }
-            if (Token::Match(tok3->tokAt(-2), "[<,] %name% <") && templateParameters(tok3) > 0)
+            if (Token::Match(tok3->tokAt(-2), "<|,|:: %name% <") && templateParameters(tok3) > 0)
                 ++indentlevel;
             else if (indentlevel > 0 && Token::Match(tok3, "> [,>]"))
                 --indentlevel;
@@ -1608,7 +1604,7 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
             if (printDebug && errorlogger) {
                 std::list<const Token *> callstack(1, tok2);
                 errorlogger->reportErr(ErrorLogger::ErrorMessage(callstack, &tokenlist, Severity::debug, "debug",
-                                       "Failed to instantiate template \"" + iter2->name + "\". The checking continues anyway.", false));
+                                       "Failed to instantiate template \"" + instantiation.name + "\". The checking continues anyway.", false));
             }
             if (typeForNewName.empty())
                 continue;
@@ -1620,12 +1616,12 @@ bool TemplateSimplifier::simplifyTemplateInstantiations(
 
         if (expandedtemplates.find(newName) == expandedtemplates.end()) {
             expandedtemplates.insert(newName);
-            TemplateSimplifier::expandTemplate(tokenlist, tok, iter2->name, typeParametersInDeclaration, newName, typesUsedInTemplateInstantiation, templateInstantiations);
+            TemplateSimplifier::expandTemplate(tokenlist, tok, instantiation.name, typeParametersInDeclaration, newName, typesUsedInTemplateInstantiation, templateInstantiations);
             instantiated = true;
         }
 
         // Replace all these template usages..
-        replaceTemplateUsage(tok2, iter2->name, typeStringsUsedInTemplateInstantiation, newName, typesUsedInTemplateInstantiation, templateInstantiations);
+        replaceTemplateUsage(tok2, instantiation.name, typeStringsUsedInTemplateInstantiation, newName, typesUsedInTemplateInstantiation, templateInstantiations);
     }
 
     // Template has been instantiated .. then remove the template declaration
@@ -1653,7 +1649,10 @@ void TemplateSimplifier::replaceTemplateUsage(Token * const instantiationToken,
     std::list<ScopeInfo2> scopeInfo;
     std::list< std::pair<Token *, Token *> > removeTokens;
     for (Token *nameTok = instantiationToken; nameTok; nameTok = nameTok->next()) {
-        setScopeInfo(nameTok, &scopeInfo);
+        if (Token::Match(nameTok, "}|namespace|class|struct")) {
+            setScopeInfo(nameTok, &scopeInfo);
+            continue;
+        }
         if (!Token::Match(nameTok, "%name% <"))
             continue;
         if (!matchTemplateParameters(nameTok, typeStringsUsedInTemplateInstantiation))
@@ -1719,7 +1718,7 @@ void TemplateSimplifier::replaceTemplateUsage(Token * const instantiationToken,
                     }
                 }
             }
-            removeTokens.push_back(std::pair<Token*,Token*>(nameTok, tok2->next()));
+            removeTokens.emplace_back(nameTok, tok2->next());
         }
 
         nameTok = tok2;
@@ -1784,32 +1783,32 @@ void TemplateSimplifier::simplifyTemplates(
             for (std::list<TokenAndName>::const_iterator iter2 = templateDeclarations.begin(); iter2 != templateDeclarations.end(); ++iter2) {
                 if (iter1->name == iter2->name) {
                     const Token *tok = iter2->token->next()->findClosingBracket();
-                    int namepos = getTemplateNamePosition(tok);
+                    const int namepos = getTemplateNamePosition(tok);
                     if (namepos > 0)
                         specializations.push_back(tok->tokAt(namepos));
                 }
             }
 
-            bool instantiated = TemplateSimplifier::simplifyTemplateInstantiations(tokenlist,
-                                errorlogger,
-                                _settings,
-                                *iter1,
-                                specializations,
-                                maxtime,
-                                templateInstantiations,
-                                expandedtemplates);
+            const bool instantiated = TemplateSimplifier::simplifyTemplateInstantiations(tokenlist,
+                                      errorlogger,
+                                      _settings,
+                                      *iter1,
+                                      specializations,
+                                      maxtime,
+                                      templateInstantiations,
+                                      expandedtemplates);
             if (instantiated)
                 instantiatedTemplates.push_back(*iter1);
         }
 
         for (std::list<TokenAndName>::const_iterator it = instantiatedTemplates.begin(); it != instantiatedTemplates.end(); ++it) {
-            std::list<TokenAndName>::iterator it1;
-            for (it1 = templateDeclarations.begin(); it1 != templateDeclarations.end(); ++it1) {
-                if (it1->token == it->token)
+            std::list<TokenAndName>::iterator decl;
+            for (decl = templateDeclarations.begin(); decl != templateDeclarations.end(); ++decl) {
+                if (decl->token == it->token)
                     break;
             }
-            if (it1 != templateDeclarations.end()) {
-                templateDeclarations.erase(it1);
+            if (decl != templateDeclarations.end()) {
+                templateDeclarations.erase(decl);
                 removeTemplate(it->token);
             }
         }
