@@ -163,6 +163,11 @@ void validCode()
     HLOCAL pLocalAlloc = LocalAlloc(1, 2);
     LocalFree(pLocalAlloc);
 
+    // cppcheck-suppress lstrlenCalled
+    (void)lstrlen(bufTC);
+    // cppcheck-suppress lstrlenCalled
+    (void)lstrlen(NULL);
+
     // Intrinsics
     __noop();
     __noop(1, "test", NULL);
@@ -485,6 +490,10 @@ void ignoredReturnValue()
 
     // cppcheck-suppress ignoredReturnValue
     _fileno(stdio);
+
+    // cppcheck-suppress lstrlenCalled
+    // cppcheck-suppress ignoredReturnValue
+    lstrlen(TEXT("test"));
 }
 
 void invalidFunctionArg()
@@ -665,3 +674,185 @@ void allocDealloc_GetModuleHandleEx()
     HMODULE hModule2;
     GetModuleHandleEx(0, NULL, &hModule2);
 }
+
+void uninitvar_tolower(_locale_t l)
+{
+    int c;
+    // cppcheck-suppress uninitvar
+    (void)_tolower(c);
+    // cppcheck-suppress uninitvar
+    (void)_tolower_l(c, l);
+}
+
+void uninitvar_toupper(_locale_t l)
+{
+    int c;
+    // cppcheck-suppress uninitvar
+    (void)_toupper(c);
+    // cppcheck-suppress uninitvar
+    (void)_toupper_l(c, l);
+}
+
+void uninitvar_towlower(_locale_t l)
+{
+    wint_t i;
+    // cppcheck-suppress uninitvar
+    (void)_towlower_l(i, l);
+}
+
+void uninitvar_towupper(_locale_t l)
+{
+    wint_t i;
+    // cppcheck-suppress uninitvar
+    (void)_towupper_l(i, l);
+}
+
+void oppositeInnerCondition_SUCCEEDED_FAILED(HRESULT hr)
+{
+    if (SUCCEEDED(hr)) {
+        // TODO ticket #8596 cppcheck-suppress oppositeInnerCondition
+        if (FAILED(hr)) {
+        }
+    }
+}
+
+/*HANDLE WINAPI CreateThread(
+  _In_opt_  LPSECURITY_ATTRIBUTES  lpThreadAttributes,
+  _In_      SIZE_T                 dwStackSize,
+  _In_      LPTHREAD_START_ROUTINE lpStartAddress,
+  _In_opt_  LPVOID                 lpParameter,
+  _In_      DWORD                  dwCreationFlags,
+  _Out_opt_ LPDWORD                lpThreadId
+);*/
+HANDLE test_CreateThread(LPSECURITY_ATTRIBUTES  lpThreadAttributes,
+                         SIZE_T                 dwStackSize,
+                         LPTHREAD_START_ROUTINE lpStartAddress,
+                         LPVOID                 lpParameter,
+                         DWORD                  dwCreationFlags,
+                         LPDWORD                lpThreadId)
+{
+    // Create uninitialized variables
+    LPSECURITY_ATTRIBUTES  uninit_lpThreadAttributes;
+    SIZE_T                 uninit_dwStackSize;
+    LPTHREAD_START_ROUTINE uninit_lpStartAddress;
+    LPVOID                 uninit_lpParameter;
+    DWORD                  uninit_dwCreationFlags;
+
+    // cppcheck-suppress leakReturnValNotUsed
+    // cppcheck-suppress uninitvar
+    (void) CreateThread(lpThreadAttributes, dwStackSize, lpStartAddress, lpParameter, uninit_dwCreationFlags, lpThreadId);
+    // cppcheck-suppress leakReturnValNotUsed
+    // cppcheck-suppress uninitvar
+    (void) CreateThread(lpThreadAttributes, dwStackSize, lpStartAddress, uninit_lpParameter, dwCreationFlags, lpThreadId);
+    // @todo uninitvar shall be reported
+    // cppcheck-suppress leakReturnValNotUsed
+    (void) CreateThread(lpThreadAttributes, dwStackSize, uninit_lpStartAddress, lpParameter, dwCreationFlags, lpThreadId);
+    // cppcheck-suppress leakReturnValNotUsed
+    // cppcheck-suppress uninitvar
+    (void) CreateThread(lpThreadAttributes, uninit_dwStackSize, lpStartAddress, lpParameter, dwCreationFlags, lpThreadId);
+    // @todo uninitvar shall be reported
+    // cppcheck-suppress leakReturnValNotUsed
+    (void) CreateThread(uninit_lpThreadAttributes, dwStackSize, lpStartAddress, lpParameter, dwCreationFlags, lpThreadId);
+
+    // cppcheck-suppress leakReturnValNotUsed
+    // cppcheck-suppress nullPointer
+    (void) CreateThread(lpThreadAttributes, dwStackSize, 0, lpParameter, dwCreationFlags, lpThreadId);
+
+    // cppcheck-suppress leakReturnValNotUsed
+    // cppcheck-suppress invalidFunctionArg
+    (void) CreateThread(lpThreadAttributes, -1, lpStartAddress, lpParameter, dwCreationFlags, lpThreadId);
+
+    // no warning shall be shown for
+    return CreateThread(lpThreadAttributes, dwStackSize, lpStartAddress, lpParameter, dwCreationFlags, lpThreadId);
+}
+
+// unsigned char *_mbscat(unsigned char *strDestination, const unsigned char *strSource);
+unsigned char * uninitvar_mbscat(unsigned char *strDestination, const unsigned char *strSource)
+{
+    unsigned char *uninit_deststr;
+    unsigned char *uninit_srcstr;
+    // cppcheck-suppress uninitvar
+    (void)_mbscat(uninit_deststr,uninit_srcstr);
+    // cppcheck-suppress uninitvar
+    (void)_mbscat(strDestination,uninit_srcstr);
+    // cppcheck-suppress uninitvar
+    (void)_mbscat(uninit_deststr,uninit_deststr);
+
+    // no warning shall be shown for
+    return _mbscat(strDestination,strSource);
+}
+
+// unsigned char *_mbscat(unsigned char *strDestination, const unsigned char *strSource);
+unsigned char * nullPointer_mbscat(unsigned char *strDestination, const unsigned char *strSource)
+{
+    // cppcheck-suppress nullPointer
+    (void)_mbscat(0,strSource);
+    // cppcheck-suppress nullPointer
+    (void)_mbscat(strDestination,0);
+
+    // no warning shall be shown for
+    return _mbscat(strDestination,strSource);
+}
+
+// errno_t _mbscat_s(unsigned char *strDestination, size_t numberOfElements, const unsigned char *strSource );
+error_t uninitvar_mbscat_s(unsigned char *strDestination, size_t numberOfElements, const unsigned char *strSource)
+{
+    unsigned char *uninit_strDestination;
+    size_t uninit_numberOfElements;
+    unsigned char *uninit_strSource;
+
+    // cppcheck-suppress uninitvar
+    (void)_mbscat_s(uninit_strDestination, numberOfElements, strSource);
+    // cppcheck-suppress uninitvar
+    (void)_mbscat_s(strDestination, uninit_numberOfElements, strSource);
+    // cppcheck-suppress uninitvar
+    (void)_mbscat_s(strDestination, numberOfElements, uninit_strSource);
+
+    // no warning shall be shown for
+    return _mbscat_s(strDestination, numberOfElements, strSource);
+}
+
+// errno_t _mbscat_s(unsigned char *strDestination, size_t numberOfElements, const unsigned char *strSource );
+error_t nullPointer_mbscat_s(unsigned char *strDestination, size_t numberOfElements, const unsigned char *strSource)
+{
+    // cppcheck-suppress nullPointer
+    (void)_mbscat_s(0, numberOfElements, strSource);
+    // cppcheck-suppress nullPointer
+    (void)_mbscat_s(strDestination, numberOfElements, 0);
+
+    // no warning shall be shown for
+    return _mbscat_s(strDestination, numberOfElements, strSource);
+}
+
+// errno_t _strncpy_s_l(char *strDest, size_t numberOfElements, const char *strSource, size_t count, _locale_t locale);
+error_t uninitvar__strncpy_s_l(char *strDest, size_t numberOfElements, const char *strSource, size_t count, _locale_t locale)
+{
+    size_t uninit_numberOfElements;
+    const char *uninit_strSource;
+    size_t uninit_count;
+    _locale_t uninit_locale;
+
+    // cppcheck-suppress uninitvar
+    (void)_strncpy_s_l(strDest, uninit_numberOfElements, strSource, count, locale);
+    // cppcheck-suppress uninitvar
+    (void)_strncpy_s_l(strDest, numberOfElements, uninit_strSource, count, locale);
+    // cppcheck-suppress uninitvar
+    (void)_strncpy_s_l(strDest, numberOfElements, strSource, uninit_count, locale);
+    // cppcheck-suppress uninitvar
+    (void)_strncpy_s_l(strDest, numberOfElements, strSource, count, uninit_locale);
+
+    // no warning shall be shown for
+    return _strncpy_s_l(strDest, numberOfElements, strSource, count, locale);
+}
+
+// errno_t _strncpy_s_l(char *strDest, size_t numberOfElements, const char *strSource, size_t count, _locale_t locale);
+error_t nullPointer__strncpy_s_l(char *strDest, size_t numberOfElements, const char *strSource, size_t count, _locale_t locale)
+{
+    // cppcheck-suppress nullPointer
+    (void)_strncpy_s_l(0, numberOfElements, strSource, count, locale);
+    // cppcheck-suppress nullPointer
+    (void)_strncpy_s_l(strDest, numberOfElements, 0, count, locale);
+
+    // no warning shall be shown for
+    return _strncpy_s_l(strDest, numberOfElements, strSource, count, locale);
+}
