@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
  */
 
 
-#include "tokenize.h"
 #include "checkstring.h"
+#include "settings.h"
 #include "testsuite.h"
-#include "testutils.h"
+#include "tokenize.h"
 
 
 class TestString : public TestFixture {
@@ -31,7 +31,7 @@ public:
 private:
     Settings settings;
 
-    void run() {
+    void run() override {
         settings.addEnabled("warning");
         settings.addEnabled("style");
 
@@ -44,7 +44,6 @@ private:
         TEST_CASE(strPlusChar1);     // "/usr" + '/'
         TEST_CASE(strPlusChar2);     // "/usr" + ch
         TEST_CASE(strPlusChar3);     // ok: path + "/sub" + '/'
-        TEST_CASE(strPlusChar4);     // ast
 
         TEST_CASE(sprintf1);        // Dangerous usage of sprintf
         TEST_CASE(sprintf2);
@@ -52,6 +51,8 @@ private:
         TEST_CASE(sprintf4);        // struct member
 
         TEST_CASE(incorrectStringCompare);
+
+        TEST_CASE(deadStrcmp);
     }
 
     void check(const char code[], const char filename[] = "test.cpp") {
@@ -89,6 +90,24 @@ private:
               "  abc[0] = 'a';\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        check("void foo_FP1(char *p) {\n"
+              "  p[1] = 'B';\n"
+              "}\n"
+              "void foo_FP2(void) {\n"
+              "  char* s = \"Y\";\n"
+              "  foo_FP1(s);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2] -> [test.cpp:5]: (error) Modifying string literal \"Y\" directly or indirectly is undefined behaviour.\n", errout.str());
+
+        check("void foo_FP1(char *p) {\n"
+              "  p[1] = 'B';\n"
+              "}\n"
+              "void foo_FP2(void) {\n"
+              "  char s[10] = \"Y\";\n"
+              "  foo_FP1(s);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void alwaysTrueFalseStringCompare() {
@@ -150,16 +169,26 @@ private:
               "{\n"
               "  if (strcmp(\"00FF00\", \"00FF00\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
+        check("void f() {\n"
+              "  if (strcmp($\"00FF00\", \"00FF00\") == 0) {}"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "  if ($strcmp(\"00FF00\", \"00FF00\") == 0) {}"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         check("int main()\n"
               "{\n"
               "  if (stricmp(\"hotdog\",\"HOTdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -168,7 +197,7 @@ private:
               "{\n"
               "  if (QString::compare(\"Hamburger\", \"Hotdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -177,7 +206,7 @@ private:
               "{\n"
               "  if (QString::compare(argv[2], \"hotdog\") == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("", errout.str());
@@ -186,7 +215,7 @@ private:
               "{\n"
               "  if (strncmp(\"hotdog\",\"hotdog\", 6) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Unnecessary comparison of static strings.\n", errout.str());
@@ -195,7 +224,7 @@ private:
               "{\n"
               "  if (strcmp(buf, buf) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Comparison of identical string variables.\n", errout.str());
@@ -204,28 +233,28 @@ private:
               "{\n"
               "  if (stricmp(buf.c_str(), buf.c_str()) == 0)"
               "  {"
-              "    std::cout << \"Equal\n\""
+              "    std::cout << \"Equal\";"
               "  }"
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Comparison of identical string variables.\n", errout.str());
 
         check("int main() {\n"
               "  if (\"str\" == \"str\") {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
         check("int main() {\n"
               "  if (\"str\" != \"str\") {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Unnecessary comparison of static strings.\n", errout.str());
 
         check("int main() {\n"
               "  if (a+\"str\" != \"str\"+b) {\n"
-              "    std::cout << \"Equal\n\"\n"
+              "    std::cout << \"Equal\";\n"
               "  }\n"
               "}");
         ASSERT_EQUALS("", errout.str());
@@ -307,10 +336,10 @@ private:
 
         // Ticket #5734
         check("int foo(char c) {\n"
-              "return c == '42';}", "test.cpp");
+              "return c == '4';}", "test.cpp");
         ASSERT_EQUALS("", errout.str());
         check("int foo(char c) {\n"
-              "return c == '42';}", "test.c");
+              "return c == '4';}", "test.c");
         ASSERT_EQUALS("", errout.str());
         check("int foo(char c) {\n"
               "return c == \"42\"[0];}", "test.cpp");
@@ -340,7 +369,7 @@ private:
 
     void suspiciousStringCompare_char() {
         check("bool foo(char* c) {\n"
-              "    return c == '\\0';\n"
+              "    return c == 'x';\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Char literal compared with pointer 'c'. Did you intend to dereference it?\n", errout.str());
 
@@ -474,11 +503,6 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void strPlusChar4() {
-        // don't crash
-        check("int test() { int +; }");
-    }
-
 
     void incorrectStringCompare() {
         check("int f() {\n"
@@ -556,6 +580,22 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
     }
+
+    void deadStrcmp() {
+        check("void f(const char *str) {\n"
+              "  if (strcmp(str, \"abc\") == 0 || strcmp(str, \"def\")) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) The expression 'strcmp(str,\"def\") != 0' is suspicious. It overlaps 'strcmp(str,\"abc\") == 0'.\n", errout.str());
+
+        check("struct X {\n"
+              "  char *str;\n"
+              "};\n"
+              "\n"
+              "void f(const struct X *x) {\n"
+              "  if (strcmp(x->str, \"abc\") == 0 || strcmp(x->str, \"def\")) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:6]: (warning) The expression 'strcmp(x->str,\"def\") != 0' is suspicious. It overlaps 'strcmp(x->str,\"abc\") == 0'.\n", errout.str());
+    }
 };
 
 REGISTER_TEST(TestString)
