@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,13 +16,17 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "settings.h"
 #include "testsuite.h"
 #include "testutils.h"
 #include "token.h"
-#include "settings.h"
+#include "tokenize.h"
+#include "tokenlist.h"
 
-#include <vector>
 #include <string>
+#include <vector>
+
+struct InternalError;
 
 
 class TestToken : public TestFixture {
@@ -38,8 +42,13 @@ private:
     std::vector<std::string> extendedOps;
     std::vector<std::string> assignmentOps;
 
-    void run() {
-        initOps();
+    void run() override {
+        arithmeticalOps = { "+", "-", "*", "/", "%", "<<", ">>" };
+        logicalOps = { "&&", "||", "!" };
+        comparisonOps = { "==", "!=", "<", "<=", ">", ">=" };
+        bitOps = { "&", "|", "^", "~" };
+        extendedOps = { ",", "[", "]", "(", ")", "?", ":" };
+        assignmentOps = { "=", "+=", "-=", "*=", "/=", "%=", "&=", "^=", "|=", "<<=", ">>=" };
 
         TEST_CASE(nextprevious);
         TEST_CASE(multiCompare);
@@ -52,6 +61,7 @@ private:
         TEST_CASE(strValue);
 
         TEST_CASE(deleteLast);
+        TEST_CASE(deleteFirst);
         TEST_CASE(nextArgument);
         TEST_CASE(eraseTokens);
 
@@ -85,16 +95,20 @@ private:
         TEST_CASE(isNameGuarantees3)
         TEST_CASE(isNameGuarantees4)
         TEST_CASE(isNameGuarantees5)
+        TEST_CASE(isNameGuarantees6)
 
         TEST_CASE(canFindMatchingBracketsNeedsOpen);
         TEST_CASE(canFindMatchingBracketsInnerPair);
         TEST_CASE(canFindMatchingBracketsOuterPair);
         TEST_CASE(canFindMatchingBracketsWithTooManyClosing);
         TEST_CASE(canFindMatchingBracketsWithTooManyOpening);
+        TEST_CASE(findClosingBracket);
+
+        TEST_CASE(expressionString);
     }
 
     void nextprevious() const {
-        Token *token = new Token(0);
+        Token *token = new Token();
         token->str("1");
         token->insertToken("2");
         token->next()->insertToken("3");
@@ -117,60 +131,58 @@ private:
     bool Match(const std::string &code, const std::string &pattern, unsigned int varid=0) {
         static const Settings settings;
         Tokenizer tokenizer(&settings, this);
-        std::istringstream istr(code);
+        std::istringstream istr(";" + code + ";");
         try {
             tokenizer.tokenize(istr, "test.cpp");
         } catch (...) {}
-        return Token::Match(tokenizer.tokens(), pattern.c_str(), varid);
+        return Token::Match(tokenizer.tokens()->next(), pattern.c_str(), varid);
     }
 
     void multiCompare() const {
         // Test for found
-        Token one(0);
+        Token one;
         one.str("one");
         ASSERT_EQUALS(1, Token::multiCompare(&one, "one|two", 0));
 
-        Token two(0);
+        Token two;
         two.str("two");
         ASSERT_EQUALS(1, Token::multiCompare(&two, "one|two", 0));
         ASSERT_EQUALS(1, Token::multiCompare(&two, "verybig|two|", 0));
 
         // Test for empty string found
-        Token notfound(0);
+        Token notfound;
         notfound.str("notfound");
-        ASSERT_EQUALS(0, Token::multiCompare(&notfound, "|one|two", 0));
-        ASSERT_EQUALS(0, Token::multiCompare(&notfound, "one||two", 0));
         ASSERT_EQUALS(0, Token::multiCompare(&notfound, "one|two|", 0));
 
         // Test for not found
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&notfound, "one|two", 0)));
 
-        Token s(0);
+        Token s;
         s.str("s");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&s, "verybig|two", 0)));
 
-        Token ne(0);
+        Token ne;
         ne.str("ne");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&ne, "one|two", 0)));
 
-        Token a(0);
+        Token a;
         a.str("a");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&a, "abc|def", 0)));
 
-        Token abcd(0);
+        Token abcd;
         abcd.str("abcd");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&abcd, "abc|def", 0)));
 
-        Token def(0);
+        Token def;
         def.str("default");
         ASSERT_EQUALS(static_cast<unsigned int>(-1), static_cast<unsigned int>(Token::multiCompare(&def, "abc|def", 0)));
 
         // %op%
-        Token plus(0);
+        Token plus;
         plus.str("+");
         ASSERT_EQUALS(1, Token::multiCompare(&plus, "one|%op%", 0));
         ASSERT_EQUALS(1, Token::multiCompare(&plus, "%op%|two", 0));
-        Token x(0);
+        Token x;
         x.str("x");
         ASSERT_EQUALS(-1, Token::multiCompare(&x, "one|%op%", 0));
         ASSERT_EQUALS(-1, Token::multiCompare(&x, "%op%|two", 0));
@@ -246,13 +258,13 @@ private:
     }
 
     void multiCompare5() const {
-        Token tok(0);
+        Token tok;
         tok.str("||");
         ASSERT_EQUALS(true, Token::multiCompare(&tok, "+|%or%|%oror%", 0) >= 0);
     }
 
     void getStrLength() const {
-        Token tok(0);
+        Token tok;
 
         tok.str("\"\"");
         ASSERT_EQUALS(0, (int)Token::getStrLength(&tok));
@@ -268,7 +280,7 @@ private:
     }
 
     void getStrSize() const {
-        Token tok(0);
+        Token tok;
 
         tok.str("\"abc\"");
         ASSERT_EQUALS(sizeof("abc"), Token::getStrSize(&tok));
@@ -281,7 +293,7 @@ private:
     }
 
     void strValue() const {
-        Token tok(0);
+        Token tok;
 
         tok.str("\"\"");
         ASSERT_EQUALS("", tok.strValue());
@@ -308,12 +320,25 @@ private:
 
 
     void deleteLast() const {
-        Token *tokensBack = 0;
-        Token tok(&tokensBack);
+        TokensFrontBack listEnds{ 0 };
+        Token **tokensBack = &(listEnds.back);
+        Token tok(&listEnds);
         tok.insertToken("aba");
-        ASSERT_EQUALS(true, tokensBack == tok.next());
+        ASSERT_EQUALS(true, *tokensBack == tok.next());
         tok.deleteNext();
-        ASSERT_EQUALS(true, tokensBack == &tok);
+        ASSERT_EQUALS(true, *tokensBack == &tok);
+    }
+
+    void deleteFirst() const {
+        TokensFrontBack listEnds{ 0 };
+        Token **tokensFront = &(listEnds.front);
+        Token tok(&listEnds);
+
+        tok.insertToken("aba");
+
+        ASSERT_EQUALS(true, *tokensFront == tok.previous());
+        tok.deletePrevious();
+        ASSERT_EQUALS(true, *tokensFront == &tok);
     }
 
     void nextArgument() const {
@@ -351,7 +376,7 @@ private:
         ASSERT_EQUALS(true, Token::Match(singleChar.tokens(), "[a|bc]"));
         ASSERT_EQUALS(false, Token::Match(singleChar.tokens(), "[d|ef]"));
 
-        Token multiChar(0);
+        Token multiChar;
         multiChar.str("[ab");
         ASSERT_EQUALS(false, Token::Match(&multiChar, "[ab|def]"));
     }
@@ -513,77 +538,33 @@ private:
     }
 
     void matchOr() const {
-        givenACodeSampleToTokenize bitwiseOr("|", true);
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "%op%"));
-        ASSERT_EQUALS(false, Token::Match(bitwiseOr.tokens(), "%oror%"));
+        givenACodeSampleToTokenize bitwiseOr(";|;", true);
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOr.tokens(), "; %op%"));
+        ASSERT_EQUALS(false, Token::Match(bitwiseOr.tokens(), "; %oror%"));
 
-        givenACodeSampleToTokenize bitwiseOrAssignment("|=");
-        ASSERT_EQUALS(false,  Token::Match(bitwiseOrAssignment.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(bitwiseOrAssignment.tokens(), "%op%"));
-        ASSERT_EQUALS(false, Token::Match(bitwiseOrAssignment.tokens(), "%oror%"));
+        givenACodeSampleToTokenize bitwiseOrAssignment(";|=;");
+        ASSERT_EQUALS(false,  Token::Match(bitwiseOrAssignment.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(bitwiseOrAssignment.tokens(), "; %op%"));
+        ASSERT_EQUALS(false, Token::Match(bitwiseOrAssignment.tokens(), "; %oror%"));
 
-        givenACodeSampleToTokenize logicalOr("||", true);
-        ASSERT_EQUALS(false, Token::Match(logicalOr.tokens(), "%or%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%op%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%oror%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "&&|%oror%"));
-        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "%oror%|&&"));
+        givenACodeSampleToTokenize logicalOr(";||;", true);
+        ASSERT_EQUALS(false, Token::Match(logicalOr.tokens(), "; %or%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %op%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %oror%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; &&|%oror%"));
+        ASSERT_EQUALS(true,  Token::Match(logicalOr.tokens(), "; %oror%|&&"));
 
-        givenACodeSampleToTokenize logicalAnd("&&", true);
-        ASSERT_EQUALS(true, Token::simpleMatch(logicalAnd.tokens(), "&&"));
-        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "&&|%oror%"));
-        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "%oror%|&&"));
+        givenACodeSampleToTokenize logicalAnd(";&&;", true);
+        ASSERT_EQUALS(true, Token::simpleMatch(logicalAnd.tokens(), "; &&"));
+        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "; &&|%oror%"));
+        ASSERT_EQUALS(true, Token::Match(logicalAnd.tokens(), "; %oror%|&&"));
     }
 
     static void append_vector(std::vector<std::string> &dest, const std::vector<std::string> &src) {
         dest.insert(dest.end(), src.begin(), src.end());
     }
 
-    void initOps() {
-        arithmeticalOps.push_back("+");
-        arithmeticalOps.push_back("-");
-        arithmeticalOps.push_back("*");
-        arithmeticalOps.push_back("/");
-        arithmeticalOps.push_back("%");
-        arithmeticalOps.push_back("<<");
-        arithmeticalOps.push_back(">>");
-
-        logicalOps.push_back("&&");
-        logicalOps.push_back("||");
-        logicalOps.push_back("!");
-        comparisonOps.push_back("==");
-        comparisonOps.push_back("!=");
-        comparisonOps.push_back("<");
-        comparisonOps.push_back("<=");
-        comparisonOps.push_back(">");
-        comparisonOps.push_back(">=");
-        bitOps.push_back("&");
-        bitOps.push_back("|");
-        bitOps.push_back("^");
-        bitOps.push_back("~");
-
-        extendedOps.push_back(",");
-        extendedOps.push_back("[");
-        extendedOps.push_back("]");
-        extendedOps.push_back("(");
-        extendedOps.push_back(")");
-        extendedOps.push_back("?");
-        extendedOps.push_back(":");
-
-        assignmentOps.push_back("=");
-        assignmentOps.push_back("+=");
-        assignmentOps.push_back("-=");
-        assignmentOps.push_back("*=");
-        assignmentOps.push_back("/=");
-        assignmentOps.push_back("%=");
-        assignmentOps.push_back("&=");
-        assignmentOps.push_back("^=");
-        assignmentOps.push_back("|=");
-        assignmentOps.push_back("<<=");
-        assignmentOps.push_back(">>=");
-    }
-
     void matchOp() {
         std::vector<std::string> test_ops;
         append_vector(test_ops, arithmeticalOps);
@@ -634,7 +615,7 @@ private:
     void isArithmeticalOp() const {
         std::vector<std::string>::const_iterator test_op, test_ops_end = arithmeticalOps.end();
         for (test_op = arithmeticalOps.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isArithmeticalOp());
         }
@@ -649,7 +630,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isArithmeticalOp(), "Failing arithmetical operator: " + *other_op);
         }
@@ -665,7 +646,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isOp());
         }
@@ -676,7 +657,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isOp(), "Failing normal operator: " + *other_op);
         }
@@ -691,7 +672,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isConstOp());
         }
@@ -703,7 +684,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isConstOp(), "Failing normal operator: " + *other_op);
         }
@@ -719,7 +700,7 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = test_ops.end();
         for (test_op = test_ops.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isExtendedOp());
         }
@@ -727,7 +708,7 @@ private:
         // Negative test against assignment operators
         std::vector<std::string>::const_iterator other_op, other_ops_end = assignmentOps.end();
         for (other_op = assignmentOps.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isExtendedOp(), "Failing assignment operator: " + *other_op);
         }
@@ -736,7 +717,7 @@ private:
     void isAssignmentOp() const {
         std::vector<std::string>::const_iterator test_op, test_ops_end = assignmentOps.end();
         for (test_op = assignmentOps.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(true, tok.isAssignmentOp());
         }
@@ -751,7 +732,7 @@ private:
 
         std::vector<std::string>::const_iterator other_op, other_ops_end = other_ops.end();
         for (other_op = other_ops.begin(); other_op != other_ops_end; ++other_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*other_op);
             ASSERT_EQUALS_MSG(false, tok.isAssignmentOp(), "Failing assignment operator: " + *other_op);
         }
@@ -760,26 +741,26 @@ private:
     void operators() const {
         std::vector<std::string>::const_iterator test_op;
         for (test_op = extendedOps.begin(); test_op != extendedOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eExtendedOp, tok.tokType());
         }
         for (test_op = logicalOps.begin(); test_op != logicalOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eLogicalOp, tok.tokType());
         }
         for (test_op = bitOps.begin(); test_op != bitOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eBitOp, tok.tokType());
         }
         for (test_op = comparisonOps.begin(); test_op != comparisonOps.end(); ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS(Token::eComparisonOp, tok.tokType());
         }
-        Token tok(nullptr);
+        Token tok;
         tok.str("++");
         ASSERT_EQUALS(Token::eIncDecOp, tok.tokType());
         tok.str("--");
@@ -787,7 +768,7 @@ private:
     }
 
     void literals() const {
-        Token tok(nullptr);
+        Token tok;
 
         tok.str("\"foo\"");
         ASSERT(tok.tokType() == Token::eString);
@@ -818,23 +799,29 @@ private:
 
         std::vector<std::string>::const_iterator test_op, test_ops_end = standard_types.end();
         for (test_op = standard_types.begin(); test_op != test_ops_end; ++test_op) {
-            Token tok(nullptr);
+            Token tok;
             tok.str(*test_op);
             ASSERT_EQUALS_MSG(true, tok.isStandardType(), "Failing standard type: " + *test_op);
         }
 
         // Negative test
-        Token tok(nullptr);
+        Token tok;
         tok.str("string");
         ASSERT_EQUALS(false, tok.isStandardType());
 
         // Change back to standard type
         tok.str("int");
         ASSERT_EQUALS(true, tok.isStandardType());
+
+        // token can't be both type and variable
+        tok.str("abc");
+        tok.isStandardType(true);
+        tok.varId(123);
+        ASSERT_EQUALS(false, tok.isStandardType());
     }
 
     void updateProperties() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("foobar");
 
         ASSERT_EQUALS(true, tok.isName());
@@ -847,7 +834,7 @@ private:
     }
 
     void updatePropertiesConcatStr() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("true");
 
         ASSERT_EQUALS(true, tok.isBoolean());
@@ -859,37 +846,42 @@ private:
     }
 
     void isNameGuarantees1() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("Name");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees2() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("_name");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees3() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("_123");
         ASSERT_EQUALS(true, tok.isName());
     }
 
     void isNameGuarantees4() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("123456");
         ASSERT_EQUALS(false, tok.isName());
         ASSERT_EQUALS(true, tok.isNumber());
     }
 
     void isNameGuarantees5() const {
-        Token tok(nullptr);
+        Token tok;
         tok.str("a123456");
         ASSERT_EQUALS(true, tok.isName());
         ASSERT_EQUALS(false, tok.isNumber());
     }
 
+    void isNameGuarantees6() const {
+        Token tok;
+        tok.str("$f");
+        ASSERT_EQUALS(true, tok.isName());
+    }
 
     void canFindMatchingBracketsNeedsOpen() const {
         givenACodeSampleToTokenize var("std::deque<std::set<int> > intsets;");
@@ -929,7 +921,24 @@ private:
         ASSERT(t != nullptr && t->str() == ">");
 
         t = var.tokens()->tokAt(4)->findClosingBracket();
-        ASSERT(t != nullptr && t->str() == ")");
+        ASSERT(t == nullptr);
+    }
+
+    void findClosingBracket() {
+        givenACodeSampleToTokenize var("template<typename X, typename...Y> struct S : public Fred<Wilma<Y...>> {}");
+
+        const Token* t = var.tokens()->next()->findClosingBracket();
+        ASSERT(Token::simpleMatch(t, "> struct"));
+    }
+
+    void expressionString() {
+        givenACodeSampleToTokenize var1("void f() { *((unsigned long long *)x) = 0; }");
+        const Token *tok1 = Token::findsimplematch(var1.tokens(), "*");
+        ASSERT_EQUALS("*((unsigned long long*)x)", tok1->expressionString());
+
+        givenACodeSampleToTokenize var2("typedef unsigned long long u64; void f() { *((u64 *)x) = 0; }");
+        const Token *tok2 = Token::findsimplematch(var2.tokens(), "*");
+        ASSERT_EQUALS("*((unsigned long long*)x)", tok2->expressionString());
     }
 };
 
